
/*==============================================================================
 *
 * (#)Src.java
 * created         2003/02/15
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *==============================================================================
 */


package com.miami.romeo.help;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.JFrame;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.ImageIcon;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreeSelectionModel;

import java.io.*;
import java.io.IOException;

import java.net.URL;

import com.miami.romeo.config.*;

public class Src extends JFrame {

  Utils utils = new Utils();

  JMenuBar menuBar;
  // Tool Bar
  private ToggleButtonToolBar toolbar = null;
  private ButtonGroup toolbarGroup = new ButtonGroup();
  private JEditorPane htmlPane;
  private static boolean DEBUG = false;
  private URL helpURL;
  private boolean playWithLineStyle = false;
  private String lineStyle = "Angled";

  public Src() {
    JPanel top0 = new JPanel();
    top0.setLayout(new BorderLayout());
    this.getContentPane().add(top0, BorderLayout.NORTH);

    // Add the JMenuBar.
    menuBar = createMenus();
    top0.add(menuBar, BorderLayout.NORTH);

    // Add the ToolBar.
    JPanel toolbarPanel = new JPanel();
    toolbarPanel.setLayout(new BorderLayout());
    toolbar = new ToggleButtonToolBar();
    toolbarPanel.add(toolbar, BorderLayout.NORTH);
    createToggleButtons();
    top0.add(toolbarPanel, BorderLayout.SOUTH);

    // Create nodes
    DefaultMutableTreeNode top = new DefaultMutableTreeNode("com");
    createNodes(top);

    // Create a jtree with single selection
    final JTree tree = new JTree(top);
    tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

    // Set icons for the nodes
    DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
    renderer.setLeafIcon(utils.createImageIcon16x16("Java_src.png"));
    renderer.setOpenIcon(utils.createImageIcon16x16("Package.png"));
    renderer.setClosedIcon(utils.createImageIcon16x16("Package.png"));
    tree.setCellRenderer(renderer);

    // Tree selection listener
    tree.addTreeSelectionListener(new TreeSelectionListener() {
      public void valueChanged(TreeSelectionEvent e) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                      tree.getLastSelectedPathComponent();
        if (node == null) return;

        Object nodeInfo = node.getUserObject();
        if (node.isLeaf()) {
          BookInfo book = (BookInfo)nodeInfo;
          displayURL(book.bookURL);
          if (DEBUG) {
            System.out.println(book.bookURL + ":  \n  ");
          }
        } else {
            displayURL(helpURL);
        }
        if (DEBUG) {
          System.out.println(nodeInfo.toString());
        }
      }
    });

    if (playWithLineStyle) {
      tree.putClientProperty("JTree.lineStyle", lineStyle);
    }

    // Create a scroll pane and add the tree
    JScrollPane treeView = new JScrollPane(tree);

    // Create the viewer
    htmlPane = new JEditorPane();
    htmlPane.setEditable(false);
    initHelp();
    JScrollPane htmlView = new JScrollPane(htmlPane);

    // Add scroll panes to the split pane
    JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    splitPane.setLeftComponent(treeView);
    splitPane.setRightComponent(htmlView);

    Dimension minimumSize = new Dimension(100, 300);
    htmlView.setMinimumSize(minimumSize);
    treeView.setMinimumSize(minimumSize);
    splitPane.setDividerLocation(290);

    splitPane.setPreferredSize(new Dimension(500, 300));

    // Add the split pane to the frame
    this.getContentPane().add(splitPane, BorderLayout.CENTER);

    JPanel statusBarPanel = new JPanel();
    statusBarPanel.setLayout(new BorderLayout());
    JTextField statusField = new JTextField();
    statusField.setEditable(false);
    statusBarPanel.add(statusField, BorderLayout.CENTER);

    this.getContentPane().add(statusBarPanel, BorderLayout.SOUTH);

    this.setTitle(utils.getString("Src.Title"));
    Image icnSrc = Toolkit.getDefaultToolkit().getImage(
                           getClass().getResource("/images/icons/16x16/Java_src.png"));
    this.setIconImage(icnSrc);
    this.setSize(800, 600);
    Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
    Rectangle frameDim = getBounds();
    this.setLocation((screenDim.width - frameDim.width) / 2,
                     (screenDim.height - frameDim.height) / 2);

    this.setVisible(true);
  }

  /**
   * Create menus
   */
  public JMenuBar createMenus() {
    JMenuItem mi;
    // ***** create the menubar ****
    Border borderMenuBar = BorderFactory.createBevelBorder(BevelBorder.RAISED,
                                                    new Color(248, 254, 255),
                                                    Color.white,
                                                    new Color(121, 124, 136),
                                                    new Color(84, 86, 95));
    JMenuBar menuBar = new JMenuBar();
    menuBar.setBorder(borderMenuBar);

    // ***** create File menu
    JMenu fileMenu = (JMenu) menuBar.add(new JMenu(utils.getString("FileMenu.file_label")));
    fileMenu.setMnemonic(utils.getMnemonic("FileMenu.file_mnemonic"));

    createMenuItem(fileMenu, "Src.extractall_label", "Src.extractall_mnemonic",
		   "Src.extractall_accessible_description", "Tgz.png", new ExtractAllAction(this));

    fileMenu.addSeparator();

    createMenuItem(fileMenu, "FileMenu.exit_label", "FileMenu.exit_mnemonic",
		   "FileMenu.exit_accessible_description", "Exit.png", new ExitAction(this));

    return menuBar;
  }

  /**
   * Creates a generic JMenuItem
   */
  public JMenuItem createMenuItem(JMenu menu, String label, String mnemonic,
			          String accessibleDescription, String filename,
				  Action action) {
    JMenuItem mi = (JMenuItem) menu.add(new JMenuItem(utils.getString(label)));
    mi.setMnemonic(utils.getMnemonic(mnemonic));
    mi.getAccessibleContext().setAccessibleDescription(utils.getString(accessibleDescription));
    mi.setIcon(utils.createImageIcon16x16(filename));
    mi.addActionListener(action);
    if(action == null) {
      mi.setEnabled(false);
    }

    return mi;
  }

  /**
   * Creates JToggleButtons.
   */
  public void createToggleButtons() {
    String[] togglebuttons = {"Tgz"};
    Action[] action = {new ExtractAllAction(this)};
    for (int i = 0; i < togglebuttons.length; i++) {
      JToggleButton tb = this.getToolBar().addToggleButton(action[i],
                                           togglebuttons[i],
					   utils.createImageIcon24x24(togglebuttons[i] + ".png"),
					   utils.getString(togglebuttons[i] +
                                           ".togglebutton_accessible_description"));
      this.getToolBarGroup().add(tb);
      if(this.getToolBarGroup().getSelection() == null) {
        tb.setSelected(true);
      }
      tb.setText(null);
      tb.setToolTipText(utils.getString(togglebuttons[i] + ".togglebutton_accessible_description"));
    }
  }

  /**
   * Returns the toolbar
   */
  public ToggleButtonToolBar getToolBar() {
    return toolbar;
  }

  /**
   * Returns the toolbar button group
   */
  public ButtonGroup getToolBarGroup() {
    return toolbarGroup;
  }

  // *******************************************************
  // **************   ToggleButtonToolbar  *****************
  // *******************************************************

  static Insets zeroInsets = new Insets(1,1,1,1);
  protected class ToggleButtonToolBar extends JToolBar {
    public ToggleButtonToolBar() {
      super();
    }

    JToggleButton addToggleButton(Action a, String tbName,
                                  ImageIcon tbIcon, String tbToolTipText) {
      JToggleButton tb = new JToggleButton(tbName, tbIcon);
      tb.setMargin(zeroInsets);
      tb.setText(null);
      tb.setEnabled(a.isEnabled());
      tb.setToolTipText(utils.getString(tbToolTipText));
      tb.setAction(a);
      tb.setIcon(tbIcon);
      add(tb);
      return tb;
    }
  }

  private class BookInfo {

    public String bookName;
    public URL bookURL;
    public String prefix = "/src/com/miami/romeo/";

    public BookInfo(String book, String filename) {
      bookName = book;
      try {
        bookURL = getClass().getResource(prefix + filename);
      }
      catch (Exception exc) {
        System.out.println("\n");
        System.err.println("Error: creation failed");
        bookURL = null;
      }
    }

    public String toString() {
      return bookName;
    }
  }

  private void initHelp() {
    String s = null;
    try {
      s = "/src/com/miami/romeo/main/Romeo.java";

      if (DEBUG) {
        System.out.println("The Help URL is " + s);
      }
      helpURL = getClass().getResource(s);
      displayURL(helpURL);
    }
    catch (Exception e) {
      System.out.println("\n");
      System.err.println("Could not create the Help URL: " + s);
    }
  }

  private void displayURL(URL url) {
    try {
      htmlPane.setPage(url);
    }
    catch (IOException e) {
    System.out.println("\n");
      System.err.println("Could not read the URL: " + url);
    }
  }

  private void createNodes(DefaultMutableTreeNode top) {
    DefaultMutableTreeNode miami = null;
    DefaultMutableTreeNode romeo = null;
    DefaultMutableTreeNode config = null;
    DefaultMutableTreeNode graphics = null;
    DefaultMutableTreeNode help = null;
    DefaultMutableTreeNode internet = null;
    DefaultMutableTreeNode main = null;
    DefaultMutableTreeNode about = null;
    DefaultMutableTreeNode combo = null;
    DefaultMutableTreeNode list = null;
    DefaultMutableTreeNode resize = null;
    DefaultMutableTreeNode table = null;
    DefaultMutableTreeNode tree = null;
    DefaultMutableTreeNode settings = null;
    DefaultMutableTreeNode themes = null;
    DefaultMutableTreeNode xmame = null;
    DefaultMutableTreeNode book = null;

    miami = new DefaultMutableTreeNode("miami");
    top.add(miami);

    romeo = new DefaultMutableTreeNode("romeo");
    miami.add(romeo);

    config = new DefaultMutableTreeNode("config");
    romeo.add(config);

       // Alphabetizer.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Alphabetizer.java",
             "config" + File.separator + "Alphabetizer.java"));
        config.add(book);

        // Array.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Array.java",
             "config" + File.separator + "Array.java"));
        config.add(book);

        // Config.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Config.java",
             "config" + File.separator + "Config.java"));
        config.add(book);

        // SortedStrings.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("SortedStrings.java",
             "config" + File.separator + "SortedStrings.java"));
        config.add(book);

        // Utils.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Utils.java",
             "config" + File.separator + "Utils.java"));
        config.add(book);

    graphics = new DefaultMutableTreeNode("graphics");
    romeo.add(graphics);
    
       // ColorSwatch.java
       book = new DefaultMutableTreeNode(new BookInfo
              ("ColorSwatch.java",
               "graphics" + File.separator + "ColorSwatch.java"));
       graphics.add(book);

       // DialogSeparator.java
       book = new DefaultMutableTreeNode(new BookInfo
              ("DialogSeparator.java",
               "graphics" + File.separator + "DialogSeparator.java"));
       graphics.add(book);
       
       // OvalBorder.java
       book = new DefaultMutableTreeNode(new BookInfo
              ("OvalBorder.java",
               "graphics" + File.separator + "OvalBorder.java"));
       graphics.add(book);

    help = new DefaultMutableTreeNode("help");
    romeo.add(help);

        // Src.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Src.java",
             "help" + File.separator + "Src.java"));
        help.add(book);

    internet = new DefaultMutableTreeNode("internet");
    romeo.add(internet);

        // Downloader.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Downloader.java",
             "internet" + File.separator + "Downloader.java"));
        internet.add(book);

        // Hyperlink.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Hyperlink.java",
             "internet" + File.separator + "Hyperlink.java"));
        internet.add(book);

        // Mailer.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Mailer.java",
             "internet" + File.separator + "Mailer.java"));
        internet.add(book);

    main = new DefaultMutableTreeNode("main");
    romeo.add(main);
    
    about = new DefaultMutableTreeNode("about");
    main.add(about);
    
        // AboutRomeo.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("AboutRomeo.java",
             "main" + File.separator + "about" + File.separator +
	     "AboutRomeo.java"));
        about.add(book);
	
	// License.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("License.java",
             "main" + File.separator + "about" + File.separator +
	     "License.java"));
        about.add(book);

	// MemoryMonitor.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("MemoryMonitor.java",
             "main" + File.separator + "about" + File.separator +
	     "MemoryMonitor.java"));
        about.add(book);

        // ScrollText.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("ScrollText.java",
             "main" + File.separator + "about" + File.separator +
	     "ScrollText.java"));
        about.add(book);
	
        // SystemInfo.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("SystemInfo.java",
             "main" + File.separator + "about" + File.separator +
	     "SystemInfo.java"));
        about.add(book);

    combo = new DefaultMutableTreeNode("combo");
    main.add(combo);
    
        // IconComboRenderer.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("IconComboRenderer.java",
             "main" + File.separator + "combo" + File.separator +
	     "IconComboRenderer.java"));
        combo.add(book);

    list = new DefaultMutableTreeNode("list");
    main.add(list);

        // InfoListCellRenderer.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("InfoListCellRenderer.java",
             "main" + File.separator + "list" + File.separator +
	     "InfoListCellRenderer.java"));
        list.add(book);
	
	// JListCellRenderer.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("JListCellRenderer.java",
             "main" + File.separator + "list" + File.separator +
	     "JListCellRenderer.java"));
        list.add(book);

	// ListItem.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("ListItem.java",
             "main" + File.separator + "list" + File.separator +
	     "ListItem.java"));
        list.add(book);
    
    resize = new DefaultMutableTreeNode("resize");
    main.add(resize);
    
        // EastResizeEdge.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("EastResizeEdge.java",
             "main" + File.separator + "resize" + File.separator +
	     "EastResizeEdge.java"));
        resize.add(book);
	
        // NorthResizeEdge.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("NorthResizeEdge.java",
             "main" + File.separator + "resize" + File.separator +
	     "NorthResizeEdge.java"));
        resize.add(book);

	// SouthResizeEdge.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("SouthResizeEdge.java",
             "main" + File.separator + "resize" + File.separator +
	     "SouthResizeEdge.java"));
        resize.add(book);
	
	// WestResizeEdge.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("WestResizeEdge.java",
             "main" + File.separator + "resize" + File.separator +
	     "WestResizeEdge.java"));
        resize.add(book);
    
    table = new DefaultMutableTreeNode("table");
    main.add(table);

        // BevelArrowIcon.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("BevelArrowIcon.java",
             "main" + File.separator + "table" + File.separator +
	     "BevelArrowIcon.java"));
        table.add(book);
	
	// BlankIcon.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("BlankIcon.java",
             "main" + File.separator + "table" + File.separator +
	     "BlankIcon.java"));
        table.add(book);
	
	// IndicatorCellRenderer.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("IndicatorCellRenderer.java",
             "main" + File.separator + "table" + File.separator +
	     "IndicatorCellRenderer.java"));
        table.add(book);

	// SortableTableModel.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("SortableTableModel.java",
             "main" + File.separator + "table" + File.separator +
	     "SortableTableModel.java"));
        table.add(book);
	
        // SortButtonRenderer.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("SortButtonRenderer.java",
             "main" + File.separator + "table" + File.separator +
	     "SortButtonRenderer.java"));
        table.add(book);
	
	// TableMap.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("TableMap.java",
             "main" + File.separator + "table" + File.separator +
	     "TableMap.java"));
        table.add(book);
	
	// TableRenderer.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("TableRenderer.java",
             "main" + File.separator + "table" + File.separator +
	     "TableRenderer.java"));
        table.add(book);
	
        // TableSorter.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("TableSorter.java",
             "main" + File.separator + "table" + File.separator +
	     "TableSorter.java"));
        table.add(book);
	
	// TableSorter2.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("TableSorter2.java",
             "main" + File.separator + "table" + File.separator +
	     "TableSorter2.java"));
        table.add(book);

    tree = new DefaultMutableTreeNode("tree");
    main.add(tree);
    
        // GraydIcon.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("GraydIcon.java",
             "main" + File.separator + "tree" + File.separator +
	     "GraydIcon.java"));
        tree.add(book);
	
	// RomsInfo.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("RomsInfo.java",
             "main" + File.separator + "tree" + File.separator +
	     "RomsInfo.java"));
        tree.add(book);
	
	// TreeNode.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("TreeNode.java",
             "main" + File.separator + "tree" + File.separator +
	     "TreeNode.java"));
        tree.add(book);
	
	// TreeNodeRenderer.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("TreeNodeRenderer.java",
             "main" + File.separator + "tree" + File.separator +
	     "TreeNodeRenderer.java"));
        tree.add(book);

        // DataIO.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("DataIO.java",
             "main" + File.separator + "DataIO.java"));
        main.add(book);

        // Games.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Games.java",
             "main" + File.separator + "Games.java"));
        main.add(book);

        // Info.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Info.java",
             "main" + File.separator + "Info.java"));
        main.add(book);

        // Options.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Options.java",
             "main" + File.separator + "Options.java"));
        main.add(book);

        // File main
        book = new DefaultMutableTreeNode(new BookInfo
            ("Romeo.java",
             "main" + File.separator + "Romeo.java"));
        main.add(book);

        // SplashWindow.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("SplashWindow.java",
             "main" + File.separator + "SplashWindow.java"));
        main.add(book);

        // UpdateDialog.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("UpdateDialog.java",
             "main" + File.separator + "UpdateDialog.java"));
        main.add(book);

        // WindowWatcher.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("WindowWatcher.java",
             "main" + File.separator + "WindowWatcher.java"));
        main.add(book);

    settings = new DefaultMutableTreeNode("settings");
    romeo.add(settings);

        // CompTitledBorder.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("CompTitledBorder.java",
             "settings" + File.separator + "CompTitledBorder.java"));
        settings.add(book);
	
        // CompTitledPane.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("CompTitledPane.java",
             "settings" + File.separator + "CompTitledPane.java"));
        settings.add(book);

        // Settings.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Settings.java",
             "settings" + File.separator + "Settings.java"));
        settings.add(book);
	
	// SettingsListCellRenderer.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("SettingsListCellRenderer.java",
             "settings" + File.separator + "SettingsListCellRenderer.java"));
        settings.add(book);
	
	// SingleRowTabbedPane.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("SingleRowTabbedPane.java",
             "settings" + File.separator + "SingleRowTabbedPane.java"));
        settings.add(book);
	
        // SingleRowTabbedPaneUI.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("SingleRowTabbedPaneUI.java",
             "settings" + File.separator + "SingleRowTabbedPaneUI.java"));
        settings.add(book);
	
	// StateTransmitter.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("StateTransmitter.java",
             "settings" + File.separator + "StateTransmitter.java"));
        settings.add(book);
	
	// StopArrowButton.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("StopArrowButton.java",
             "settings" + File.separator + "StopArrowButton.java"));
        settings.add(book);
	
	// TabbedPaneButton.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("TabbedPaneButton.java",
             "settings" + File.separator + "TabbedPaneButton.java"));
        settings.add(book);


    themes = new DefaultMutableTreeNode("themes");
    romeo.add(themes);

        // AquaTheme.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("AquaTheme.java",
             "themes" + File.separator + "AquaTheme.java"));
        themes.add(book);

        // CharcoalTheme.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("CharcoalTheme.java",
             "themes" + File.separator + "CharcoalTheme.java"));
        themes.add(book);

        // RomeoTheme.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("RomeoTheme.java",
             "themes" + File.separator + "RomeoTheme.java"));
        themes.add(book);

        // ContrastTheme.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("ContrastTheme.java",
             "themes" + File.separator + "ContrastTheme.java"));
        themes.add(book);

        // EmeraldTheme.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("EmeraldTheme.java",
             "themes" + File.separator + "EmeraldTheme.java"));
        themes.add(book);

        // RubyTheme.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("RubyTheme.java",
             "themes" + File.separator + "RubyTheme.java"));
        themes.add(book);

    xmame = new DefaultMutableTreeNode("xmame");
    romeo.add(xmame);

        // Xmame.java
        book = new DefaultMutableTreeNode(new BookInfo
            ("Xmame.java",
             "xmame" + File.separator + "Xmame.java"));
        xmame.add(book);
  }

  // *******************************************************
  // ********************   Actions  ***********************
  // *******************************************************

  /**
   * ExtractAll Action
   */
  class ExtractAllAction extends AbstractAction {
    Src src;
    String saveDir;
    protected ExtractAllAction(Src src) {
      super("ExitAction");
      this.src = src;
    }

    public void actionPerformed(ActionEvent e) {
      String[] srcFiles = {"/src/com/miami/romeo/config/Alphabetizer.java",
                           "/src/com/miami/romeo/config/Array.java",
                           "/src/com/miami/romeo/config/Config.java",
                           "/src/com/miami/romeo/config/SortedStrings.java",
			   "/src/com/miami/romeo/config/Utils.java",
			   "/src/com/miami/romeo/graphics/ColorSwatch.java",
			   "/src/com/miami/romeo/graphics/DialogSeparator.java",
			   "/src/com/miami/romeo/graphics/OvalBorder.java",
			   "/src/com/miami/romeo/help/Src.java",
			   "/src/com/miami/romeo/internet/Downloader.java",
			   "/src/com/miami/romeo/internet/Hyperlink.java",
			   "/src/com/miami/romeo/internet/Mailer.java",
			   "/src/com/miami/romeo/main/about/AboutRomeo.java",
                           "/src/com/miami/romeo/main/about/License.java",
			   "/src/com/miami/romeo/main/about/MemoryMonitor.java",
			   "/src/com/miami/romeo/main/about/ScrollText.java",
			   "/src/com/miami/romeo/main/about/SystemInfo.java",
			   "/src/com/miami/romeo/main/combo/IconComboRenderer.java",
			   "/src/com/miami/romeo/main/list/InfoListCellRenderer.java",
			   "/src/com/miami/romeo/main/list/JListCellRenderer.java",
			   "/src/com/miami/romeo/main/list/ListItem.java",
			   "/src/com/miami/romeo/main/resize/EastResizeEdge.java",
			   "/src/com/miami/romeo/main/resize/NorthResizeEdge.java",
			   "/src/com/miami/romeo/main/resize/SouthResizeEdge.java",
			   "/src/com/miami/romeo/main/resize/WestResizeEdge.java",
			   "/src/com/miami/romeo/main/table/BevelArrowIcon.java",
			   "/src/com/miami/romeo/main/table/BlankIcon.java",
			   "/src/com/miami/romeo/main/table/IndicatorCellRenderer.java",
			   "/src/com/miami/romeo/main/table/SortableTableModel.java",
			   "/src/com/miami/romeo/main/table/SortButtonRenderer.java",
			   "/src/com/miami/romeo/main/table/TableMap.java",
			   "/src/com/miami/romeo/main/table/TableRenderer.java",
			   "/src/com/miami/romeo/main/table/TableSorter.java",
			   "/src/com/miami/romeo/main/table/TableSorter2.java",
			   "/src/com/miami/romeo/main/tree/GraydIcon.java",
			   "/src/com/miami/romeo/main/tree/RomsInfo.java",
			   "/src/com/miami/romeo/main/tree/TreeNode.java",
			   "/src/com/miami/romeo/main/tree/TreeNodeRenderer.java",
			   "/src/com/miami/romeo/main/DataIO.java",
			   "/src/com/miami/romeo/main/Games.java",
                           "/src/com/miami/romeo/main/Info.java",
                           "/src/com/miami/romeo/main/Options.java",
			   "/src/com/miami/romeo/main/Romeo.java",
			   "/src/com/miami/romeo/main/SplashWindow.java",
			   "/src/com/miami/romeo/main/UpdateDialog.java",
			   "/src/com/miami/romeo/main/WindowWatcher.java",
			   "/src/com/miami/romeo/settings/CompTitledBorder.java",
			   "/src/com/miami/romeo/settings/CompTitledPane.java",
			   "/src/com/miami/romeo/settings/Settings.java",
			   "/src/com/miami/romeo/settings/SettingsListCellRenderer.java",
			   "/src/com/miami/romeo/settings/SingleRowTabbedPane.java",
			   "/src/com/miami/romeo/settings/SingleRowTabbedPaneUI.java",
			   "/src/com/miami/romeo/settings/StateTransmitter.java",
			   "/src/com/miami/romeo/settings/StopArrowButton.java",
			   "/src/com/miami/romeo/settings/TabbedPaneButton.java",
			   "/src/com/miami/romeo/themes/AquaTheme.java",
			   "/src/com/miami/romeo/themes/CharcoalTheme.java",
			   "/src/com/miami/romeo/themes/ContrastTheme.java",
			   "/src/com/miami/romeo/themes/EmeraldTheme.java",
			   "/src/com/miami/romeo/themes/RubyTheme.java",
			   "/src/com/miami/romeo/xmame/Xmame.java"};

      //Create a file chooser
      JFileChooser fc = new JFileChooser();
      fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
      int returnVal = fc.showSaveDialog(Src.this);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        saveDir = fc.getSelectedFile().getPath();
      }

      String prefixDir = saveDir + File.separator + "src" +
                         File.separator + "com" + File.separator +
			 "miami" + File.separator + "romeo" + File.separator;

      File[] dirs = {new File(saveDir + File.separator + "src"),
                     new File(saveDir + File.separator + "src" +
		              File.separator + "com"),
		     new File(saveDir + File.separator + "src" +
		              File.separator + "com" + File.separator +
			      "miami"),
		     new File(saveDir + File.separator + "src" +
		              File.separator + "com" + File.separator +
			      "miami" + File.separator + "romeo"),
                     new File(prefixDir + "config"),
		     new File(prefixDir + "graphics"),
                     new File(prefixDir + "help"),
		     new File(prefixDir + "internet"),
		     new File(prefixDir + "main"),
                     new File(prefixDir + "main" + File.separator + "about"),
		     new File(prefixDir + "main" + File.separator + "combo"),
		     new File(prefixDir + "main" + File.separator + "list"),
		     new File(prefixDir + "main" + File.separator + "resize"),
		     new File(prefixDir + "main" + File.separator + "table"),
		     new File(prefixDir + "main" + File.separator + "tree"),
		     new File(prefixDir + "themes"),
		     new File(prefixDir + "settings"),
		     new File(prefixDir + "xmame")};

      for (int i = 0; i < dirs.length; i++) {
        if (!dirs[i].exists())
	  dirs[i].mkdir();
      }

      File[] savedFiles = {new File(prefixDir + "config" + File.separator + "Alphabetizer.java"),
                           new File(prefixDir + "config" + File.separator + "Array.java"),
                           new File(prefixDir + "config" + File.separator + "Config.java"),
                           new File(prefixDir + "config" + File.separator + "SortedStrings.java"),
			   new File(prefixDir + "config" + File.separator + "Utils.java"),
			   new File(prefixDir + "graphics" + File.separator + "ColorSwatch.java"),
			   new File(prefixDir + "graphics" + File.separator + "DialogSeparator.java"),
			   new File(prefixDir + "graphics" + File.separator + "OvalBorder.java"),
			   new File(prefixDir + "help" + File.separator + "Src.java"),
			   new File(prefixDir + "internet" + File.separator + "Downloader.java"),
			   new File(prefixDir + "internet" + File.separator + "Hyperlink.java"),
			   new File(prefixDir + "internet" + File.separator + "Mailer.java"),
			   new File(prefixDir + "main" + File.separator + "about" + File.separator + "AboutRomeo.java"),
                           new File(prefixDir + "main" + File.separator + "about" + File.separator + "License.java"),
			   new File(prefixDir + "main" + File.separator + "about" + File.separator + "MemoryMonitor.java"),
			   new File(prefixDir + "main" + File.separator + "about" + File.separator + "ScrollText.java"),
			   new File(prefixDir + "main" + File.separator + "about" + File.separator + "SystemInfo.java"),
			   new File(prefixDir + "main" + File.separator + "combo" + File.separator + "IconComboRenderer.java"),
			   new File(prefixDir + "main" + File.separator + "list" + File.separator + "InfoListCellRenderer.java"),
			   new File(prefixDir + "main" + File.separator + "list" + File.separator + "JListCellRenderer.java"),
			   new File(prefixDir + "main" + File.separator + "list" + File.separator + "ListItem.java"),
			   new File(prefixDir + "main" + File.separator + "resize" + File.separator + "EastResizeEdge.java"),
			   new File(prefixDir + "main" + File.separator + "resize" + File.separator + "NorthResizeEdge.java"),
			   new File(prefixDir + "main" + File.separator + "resize" + File.separator + "SouthResizeEdge.java"),
			   new File(prefixDir + "main" + File.separator + "resize" + File.separator + "WestResizeEdge.java"),
			   new File(prefixDir + "main" + File.separator + "table" + File.separator + "BevelArrowIcon.java"),
			   new File(prefixDir + "main" + File.separator + "table" + File.separator + "BlankIcon.java"),
			   new File(prefixDir + "main" + File.separator + "table" + File.separator + "IndicatorCellRenderer.java"),
			   new File(prefixDir + "main" + File.separator + "table" + File.separator + "SortableTableModel.java"),
			   new File(prefixDir + "main" + File.separator + "table" + File.separator + "SortButtonRenderer.java"),
			   new File(prefixDir + "main" + File.separator + "table" + File.separator + "TableMap.java"),
			   new File(prefixDir + "main" + File.separator + "table" + File.separator + "TableRenderer.java"),
			   new File(prefixDir + "main" + File.separator + "table" + File.separator + "TableSorter.java"),
			   new File(prefixDir + "main" + File.separator + "table" + File.separator + "TableSorter2.java"),
			   new File(prefixDir + "main" + File.separator + "tree" + File.separator + "GraydIcon.java"),
			   new File(prefixDir + "main" + File.separator + "tree" + File.separator + "RomsInfo.java"),
			   new File(prefixDir + "main" + File.separator + "tree" + File.separator + "TreeNode.java"),
			   new File(prefixDir + "main" + File.separator + "tree" + File.separator + "TreeNodeRenderer.java"),
			   new File(prefixDir + "main" + File.separator + "DataIO.java"),
			   new File(prefixDir + "main" + File.separator + "Games.java"),
                           new File(prefixDir + "main" + File.separator + "Info.java"),
                           new File(prefixDir + "main" + File.separator + "Options.java"),
			   new File(prefixDir + "main" + File.separator + "Romeo.java"),
			   new File(prefixDir + "main" + File.separator + "SplashWindow.java"),
			   new File(prefixDir + "main" + File.separator + "UpdateDialog.java"),
			   new File(prefixDir + "main" + File.separator + "WindowWatcher.java"),
			   new File(prefixDir + "settings" + File.separator + "CompTitledBorder.java"),
			   new File(prefixDir + "settings" + File.separator + "CompTitledPane.java"),
			   new File(prefixDir + "settings" + File.separator + "Settings.java"),
			   new File(prefixDir + "settings" + File.separator + "SettingsListCellRenderer.java"),
			   new File(prefixDir + "settings" + File.separator + "SingleRowTabbedPane.java"),
			   new File(prefixDir + "settings" + File.separator + "SingleRowTabbedPaneUI.java"),
			   new File(prefixDir + "settings" + File.separator + "StateTransmitter.java"),
			   new File(prefixDir + "settings" + File.separator + "StopArrowButton.java"),
			   new File(prefixDir + "settings" + File.separator + "TabbedPaneButton.java"),
			   new File(prefixDir + "themes" + File.separator + "AquaTheme.java"),
			   new File(prefixDir + "themes" + File.separator + "CharcoalTheme.java"),
			   new File(prefixDir + "themes" + File.separator + "ContrastTheme.java"),
			   new File(prefixDir + "themes" + File.separator + "EmeraldTheme.java"),
			   new File(prefixDir + "themes" + File.separator + "RubyTheme.java"),
			   new File(prefixDir + "xmame" + File.separator + "Xmame.java")};

      for (int i = 0; i < savedFiles.length; i++) {
        // open data
        URL url = getClass().getResource(srcFiles[i]);

        try {
	  // convert url to buffered string
	  InputStream is = url.openStream();
	  InputStreamReader isr = new InputStreamReader(is);
	  BufferedReader reader = new BufferedReader(isr);
	  BufferedWriter writer = new BufferedWriter(new FileWriter(savedFiles[i]));

	  // read one line at a time
	  String line = reader.readLine();
	  while(line != null) {
	    writer.write(line + "\n");

	    line = reader.readLine();
	  }

	  writer.close();
	  reader.close();
	  isr.close();
	  is.close();
	}
	catch(IOException ioe) {
	  System.err.println(ioe.getMessage());
	}
      }
    }
  }

  /**
   * Exit Action
   */
  class ExitAction extends AbstractAction {
    Src src;
    protected ExitAction(Src src) {
      super("ExitAction");
      this.src = src;
    }

    public void actionPerformed(ActionEvent e) {
      dispose();
    }
  }
}

 /*
  * Fine di Src.java
  */




