
/*==============================================================================
 *
 * (#)Games.java
 * created      2003/03/25
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *==============================================================================
 */

package com.miami.romeo.main;

//****** Imports
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.ArrayList;
import java.io.*;
import java.beans.*;
import java.net.*;
import java.text.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

// JTree
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.TreePath;

import javax.swing.plaf.basic.*;

// JTable
import javax.swing.table.*;

// Jlist
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import javax.swing.DefaultListModel;

import javax.swing.text.*;

import javax.swing.colorchooser.*;
import javax.swing.filechooser.FileFilter;

import javax.accessibility.*;

import com.miami.romeo.config.*;
import com.miami.romeo.xmame.*;
import com.miami.romeo.internet.Downloader;
import com.miami.romeo.main.tree.*;
import com.miami.romeo.main.table.*;
import com.miami.romeo.main.list.*;
import com.miami.romeo.main.combo.*;
import com.miami.romeo.graphics.*;

/**
 * Games.<br>
 * Panel with a single selection tree, the list of games and
 * a message area to show any errors or diagnostics.
 *
 * @author       $Author: Stefano Bolli
 * @version      $Revision: 1.23 $Date: 2004/01/04 00:44:35 $
 */
public class Games extends JPanel {
  //******* Classes needed by Games.java
  // Main class
  Romeo romeo           = null;
  // Utilities
  Utils utils           = new Utils();
  // Class to read and write configuration files
  Config config         = null;
  // Class to handle I/O operations
  DataIO dataio = null;
  // Class to dialog with xmame
  Xmame xmame           = null;
  // Downloader window
  Downloader downloader = null;
  // This class creates a button icon
  ColorSwatch cs        = null;
  // This is a JDialog to show saving operation progress
  UpdateDialog updateDialog = null;
  // Stuff to list info about a game
  JDialog infoDialog    = null;

  //******* Combo Box to list xmame executables
  // Romeo supports multiple xmame executables since version 0.1.3pre1
  public JComboBox xmameComboBox = null;

  //******* Single selection tree
  JPanel leftPanel                     = null;
  JPanel xmameComboBoxPanel            = null;
  DialogSeparator ds                   = null;
  JPanel treePanel                     = null;
  public JTree tree                    = null;
  private boolean DEBUG                = false;
  private URL romsURL;
  RomsInfo romsinfo                    = null;
  String selection;
  private boolean playWithLineStyle    = false;
  private String lineStyle             = "Angled";
  public TreeNodeRenderer treeRenderer = null;

  //******** Tables stuff
  // Tables options
  public JLabel  interCellSpacingLabel         = null;
  public JSlider interCellSpacingSlider        = null;
  public JLabel rowHeightLabel                 = null;
  public JSlider rowHeightSlider               = null;
  public JCheckBox showHorizontalLinesCheckBox = null;
  public JCheckBox showVerticalLinesCheckBox   = null;
  public JButton intTablesColor                = null;
  public JButton extTablesColor                = null;
  
  // Table rows
  public Vector row;
  // Game name
  String gameName;
  // Is the selected game original or a clone?
  String cloneOf;
  // Game driver
  String driver;
  // Data to populate table rows
  String available, description, name, year, manufacturer, icon,
         working, clone, favorite, category, sample, played, path;
  // Messages for the user
  String message;
  // Empty array containing table column names
  Vector columnNames = null;
  // Empty vector of rows
  Vector dataRows = null;
  // Array containing info about roms
  Object[] datarows = new Object[0];
  Object[] rowLine  = new Object[0];
  // Yes :|
  String yes = "Yes";

  // Column models
  public TableColumnModel columnModel1;
  public TableColumnModel columnModel2;
  public TableColumnModel columnModel3;
  // Table renderers
  public TableRenderer renderer          = null;
  public IndicatorCellRenderer iRenderer = null;
  public SortButtonRenderer sbr          = null;
  // Panel containing table options
  public JPanel tableOptionsPanel = null;
  public JPanel rightPanel        = null;
  JScrollBar headScroll;
  JScrollPane outerPane;
  JViewport innerPort;
  Vector tables;
  JPanel headerPanel;
  JViewport headerPort1 = null;;
  JViewport headerPort2 = null;;
  JViewport headerPort3 = null;;
  JViewport outerPort   = null;;
  TableModel model      = null;
  // Class to index tables rows in alphabetical order
  TableSorter sorter    = null;
  // Tables
  public JTable table   = null;
  public JTable table2  = null;
  public JTable table3  = null;
  JPanel scrollPanel    = null;
  // Popup menu for the tables
  JMenu infoMenu        = null;
  JPopupMenu popup      = null;

  //****** message area stuff
  // Format the date
  SimpleDateFormat formatter;
  // String to show the date
  String lastDate;
  // Date
  Date currentDate;
  // message area
  public JTextPane messageArea      = null;
  public JScrollPane scrollMessages = null;
  public JPanel messageAreaPanel    = null;
  
  // Split panes
  public JSplitPane horzSplitPane = null;
  public JSplitPane vertSplitPane = null;
  
  JFrame frame = null;

  /**
   * The constructor initializes all components.
   *
   * @param romeo The caller
   */
  public Games(Romeo romeo) {
    this.romeo = romeo;
    this.frame = romeo.getFrame();

    /*
     Now, we' ll create a panel in this way:

            ____________________________________
	   | combo box |scrollable table headers|
	   |___________|________________________|
	   |           |      |          |      |
	   |           |      |          |      |
	   |           |      |          |      |
	   |  single   |      |          |      |
	   | selection |table1|  table2  |table3|
	   |   tree    |      |          |      |
	   |           |      |          |      |
	   |           |      |          |      |
	   |___________|______|__________|______|
	   |            message area            |
           |____________________________________|
	   
     The combo box will list the xmame executables the user wanna use.
     Next, we' ll have tree tables: table1 and table3 will be fixed,
     (only vert. scroll), table2 will be scrollable horizontally and
     vertically. table2 will be scrollable behind table1 and table3.
     The selection of a tree node will show the list of games in the
     tables. Then we' ll create a message area for debug on the bottom.
    */

    // Call Config to read the configuration file
    config = new Config(new File(System.getProperty("user.home") +
                                 File.separator + ".romeo4xmame" +
				 File.separator + "Romeo.ini"));
				 
    // *******************************************************
    // ****************  Horz. Splitpane   *******************
    // *******************************************************

    // Create a first horizontal splitpane
    horzSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    horzSplitPane.setContinuousLayout(true);
    
    // horizontal split pane: divider location
    horzSplitPane.setDividerLocation(config.getInt("horz_split_location"));

    // *******************************************************
    // *******************  Left Panel  **********************
    // *******************************************************

    JPanel leftPanel = new JPanel(new BorderLayout());

    //******* Panel with the combobox to list xmame executable files
    xmameComboBoxPanel = new JPanel(new BorderLayout());
    //xmameComboBoxPanel.setBorder(new OvalBorder());
    int panelWidth = (int)xmameComboBoxPanel.getWidth();
    xmameComboBoxPanel.setPreferredSize(new Dimension(panelWidth, 55));
    xmameComboBoxPanel.setBorder(new EmptyBorder(5, 10, 5, 10));

    // Label for the combobox
    JLabel xmameComboBoxLabel = new JLabel(" " + utils.getString(
                                       "xmame.choose_executable_label") + " ");
    xmameComboBoxLabel.setBorder(new EmptyBorder(0, 0, 5, 10));
    xmameComboBoxPanel.add(xmameComboBoxLabel, BorderLayout.CENTER);

    // Add the combobox to list xmame executables
    xmameComboBoxPanel.add(createXmameComboBox(), BorderLayout.SOUTH);

    // Add the panel created above
    leftPanel.add(xmameComboBoxPanel, BorderLayout.NORTH);

    //******* Panel containing a single selection tree
    treePanel = new JPanel(new BorderLayout());

    // Create and add the selection tree
    treePanel.add(createTree(), BorderLayout.CENTER);
    leftPanel.add(treePanel, BorderLayout.CENTER);

    // Add the panel to the horizontal split pane on the left
    horzSplitPane.setLeftComponent(leftPanel);

    // *******************************************************
    // *******************  Right Panel **********************
    // *******************************************************

    //******* Add tables options and tables to the split pane
    rightPanel = new JPanel(new BorderLayout());

    // Create a panel with the table options
    tableOptionsPanel = createTableOptions();
    // Does the user wanna see the tables options?
    if (config.getBoolean("Tables.show_options"))
      rightPanel.add(tableOptionsPanel, BorderLayout.NORTH);

    // Create and add tables to the right panel
    rightPanel.add(createTables(), BorderLayout.CENTER);

    // Add right panel to the horizontal splitpane on the right
    horzSplitPane.setRightComponent(rightPanel);
    
    JPanel topPanel = new JPanel(new BorderLayout());
    topPanel.add(horzSplitPane, BorderLayout.CENTER);

    // *******************************************************
    // **************** Vertical Splitpane *******************
    // *******************************************************

    // Create a vertical split pane
    vertSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    //vertSplitPane.setOneTouchExpandable(true);
    vertSplitPane.setContinuousLayout(true);

    // Add the horizontal splitpane to the vertical split pane on the top
    vertSplitPane.setLeftComponent(topPanel);

    // Panel containing the message area
    messageAreaPanel = new JPanel(new BorderLayout());
    //messageAreaPanel.setBorder(new OvalBorder());

    // Create the scrollable message area
    scrollMessages = createMessageArea();
    messageAreaPanel.add(scrollMessages, BorderLayout.CENTER);

    // Verify if the user wanna see the message area
    if (config.getBoolean("View.messageArea"))
      // if true, add the message area on the bottom
      vertSplitPane.setRightComponent(messageAreaPanel);

    int frameHeight = config.getInt("Romeo.SIZE_Y");
    // Divider location for the vertical split pane
    vertSplitPane.setDividerLocation(frameHeight - (frameHeight / 3));

    setLayout(new BorderLayout());
    setBorder(new EtchedBorder());
    add(vertSplitPane, BorderLayout.CENTER);

    // Since version 0.1.3pre1 Romeo is less "1024x768" friendly.
    // This interface resizes any components according to the frame
    // size.
    rightPanel.addComponentListener(new ComponentAdapter(){
      public void componentMoved(ComponentEvent e){
        // not needed!
      }

      public void componentResized(ComponentEvent e) {
	resize(rightPanel);
      }
    });
  }

  // *******************************************************
  // ********************  Combo box  **********************
  // *******************************************************

  /**
   * This method creates a combo box to list available xmame 
   * executables and then adds it to a scroll pane.
   *
   * @return <code>JComboBox</code>
   *
   * @see    #checkXmame(String)
   *
   * @since  0.1.3pre1
   */
  public JComboBox createXmameComboBox() {
    // Number of executable to list
    int numberOfXmame = config.getInt("xmame.number_of_executable_files");

    // Items: paths to xmame executables set by the user
    String[] xmamePaths;
    
    // Check if number of items is > 0
    if (numberOfXmame > 0) {
      xmamePaths = utils.parseValues(config.get("xmame.executable_file"), ';');
    }
    else {
      xmamePaths = new String[1];
      xmamePaths[0] = "<NOT SET>";
    }

    // Create a new JComboBox with items
    xmameComboBox = new JComboBox(xmamePaths);

    int defaultIndex = config.getInt("xmame.default_executable_file");

    if (defaultIndex > 0)
      // Set selected index
      xmameComboBox.setSelectedIndex(defaultIndex);
      
    xmameComboBox.setRenderer(new IconComboRenderer(xmameComboBox));

    xmame = new Xmame(romeo);
    
    // Add an action listener
    xmameComboBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // Get the source
	JComboBox cb = (JComboBox)e.getSource();

	// Get the selected xmame executable path
	String xmamePath = cb.getSelectedItem().toString();

	// Perform some operations when a xmame executable has been
	// selected
	checkXmame(xmamePath);
      }
    });

    return xmameComboBox;
  }

  // *******************************************************
  // ********************    Tree    ***********************
  // *******************************************************

  /**
   * This method create a single selection JTree and then adds it
   * to a JScrollPane.
   *
   * @return <code>JScrollPane</code>
   *
   * @see    #createNodes(TreeNode)
   * @see    #showListOfGames(String)
   * @see    #checkCatver()
   *
   * @since  0.1a
   */
  public JScrollPane createTree() {
    // Create the nodes.
    TreeNode top = new TreeNode(utils.getString("Tree.games_label"));

    // create nodes
    createNodes(top);

    // Create a single selection tree
    tree = new JTree(top);

    int r = config.getInt("Tree.background_red");
    int g = config.getInt("Tree.background_green");
    int b = config.getInt("Tree.background_blue");

    if (r > -1 && g > -1 && b > -1) {
      tree.setBackground(new Color(r, g, b));
    }
    else {
      tree.setBackground(Color.white);
    }

    // Set selection mode: single selection
    tree.getSelectionModel().setSelectionMode(
                                    TreeSelectionModel.SINGLE_TREE_SELECTION);

    // Create a renderer for the nodes
    treeRenderer = new TreeNodeRenderer(new Color(r, g, b));
    tree.setCellRenderer(treeRenderer);
    tree.setEditable(false);

    // Listen for selection changes
    tree.addTreeSelectionListener(new TreeSelectionListener() {
      public void valueChanged(TreeSelectionEvent e) {
        TreeNode node = (TreeNode)tree.getLastSelectedPathComponent();

        if (node == null) return;

        Object nodeInfo = node.getUserObject();

	if (node.isLeaf()) {
	  romsinfo = (RomsInfo)nodeInfo;
          showListOfGames(romsinfo.romsURL);
          if (DEBUG) {
            //System.out.println(romsinfo.romsURL + ": \n ");
          }
	}
	if (DEBUG) {
	  System.out.println(romsinfo.romsURL + ": \n ");
	}
      }
    });

    if (playWithLineStyle) {
      tree.putClientProperty("JTree.lineStyle", lineStyle);
    }

    String xmamePath = getXmamePath();
    File xmameExecutable = new File(xmamePath);

    if ((!xmamePath.equals("<NOT SET>")) && (xmameExecutable.exists())) {
      tree.setEnabled(true);
    }
    else {
      tree.setEnabled(false);
    }

    checkCatver();

    // Add a mouse listener to act on tree row right-click.
    MouseListener l = new MouseListener() {
      // Needed by this interface
      public void mouseEntered (MouseEvent e) {
        // to handle
      }

      // Needed by this interface
      public void mousePressed(MouseEvent e) {
        // to handle
      }

      // Needed by this interface
      public void mouseClicked(MouseEvent e) {
        // to handle
      }

      /* Right click shows a popup menu */
      public void mouseReleased(MouseEvent e) {
        JTree treeSource = (JTree)e.getSource();

        if (e.isShiftDown()|| e.isControlDown()|| e.isAltDown()) {
          return;
        }
        if (e.isMetaDown()) {
          int x = e.getX();
          int y = e.getY();

          createTreePopupMenu(treeSource, x, y);
        }
      }

      // Needed by this interface
      public void mouseExited(MouseEvent e) {
        // to handle
      }
    };
    
    tree.addMouseListener(l);
    
    // Create a scroll pane and add the tree to it
    JScrollPane scrollTree = new JScrollPane(tree);

    return scrollTree;
  }

  /**
   * This methods creates tree nodes.
   *
   * @param top The root tree node
   */
  protected void createNodes(TreeNode top) {
    TreeNode roms = null;

      // All games
      roms = new TreeNode(new RomsInfo(
                                utils.getString("Tree.games.all_label"),
    		                "all"));

      top.add(roms);

      // Favorite games
      roms = new TreeNode(new RomsInfo(
                                utils.getString("Tree.games.favorite_label"),
		                "favorite"));

      top.add(roms);

      // Available games
      roms = new TreeNode(new RomsInfo(
                                utils.getString("Tree.games.available_label"),
		                "available"));

      top.add(roms);

      // Original games
      roms = new TreeNode(new RomsInfo(
                                utils.getString("Tree.games.original_label"),
			        "original"));

      top.add(roms);

      // Cloned games
      roms = new TreeNode(new RomsInfo(
                                utils.getString("Tree.games.cloned_label"),
			        "cloned"));

      top.add(roms);

      // Working games
      roms = new TreeNode(new RomsInfo(
                                utils.getString("Tree.games.working_label"),
			        "working"));

      top.add(roms);

      // Non-working games
      roms = new TreeNode(new RomsInfo(
                                utils.getString("Tree.games.nonworking_label"),
			        "trash"));

      top.add(roms);
      
      // Neo-Geo games
      roms = new TreeNode(new RomsInfo("Neo-Geo", "neogeo"));

      top.add(roms);
      
      // CPS1 games
      roms = new TreeNode(new RomsInfo("CPS1", "cps1"));

      top.add(roms);
      
      // CPS2 games
      roms = new TreeNode(new RomsInfo("CPS2", "cps2"));

      top.add(roms);

      // Category 'roms per year'
      TreeNode category1 = new TreeNode(
                                utils.getString("Tree.games.peryear_label"));

      top.add(category1);

      // Add years from 1975 to 2002
      for (int i = 1975; i < 2002; i++) {
        roms = new TreeNode(new RomsInfo(
                                    i + " ",
				    "year/" + i));

        category1.add(roms);
      }

      // Category 'Manufacturers'
      TreeNode category2 = new TreeNode(
                                  utils.getString(
                                  "Tree.games.manufacturer_label"));

      top.add(category2);

      // Manufacturers
      String[] manufacturer = {"<unknown>", "8ing", "Able Corp, Ltd.", "ABM & Gecas",
                               "AC", "Academysoft-Elory", "ADK", "Afega", "Aicom",
                               "Allied Leisure", "Allumer", "Alpha Denshi Co.", "Amenip",
                               "American Sammy Corp.", "Amstar", "Amutech", "Apple",
                               "Armenia", "Artic", "Associated Overseas MFR, Inc", "Atari", 
			       "Athena", "Atlus", "ATW", "ATW USA, Inc.", "Bally Midway",
			       "Banpresto", "Bee-Oh", "bootleg", "BROOKS", "Capcom", "Central Denshi",
                               "Centuri", "Chuo Co. Ltd", "Cinematronics", "Comad",
                               "Comsoft", "Cream", "CVS", "D. R. Korea", "Daiichi Denshi",
			       "Data East Corporation", "Dat Japan", "DECO", "Deniam", "Denki Onkyo",
			       "DIGITAL SOFT", "Digitrex Techstar", "Dongsung", "Dooyong", "Dorneer", 
			       "Du Tech", "Dyna Electronics", "Dynamic PI. Toei Animation",
			       "Dynax", "E.G. Felaco", "East Technology", "Eastern Micro Electronics, Inc.", 
			       "Ecogames S.L. Spain", "Ed", "Eighting", "Elcon",
                               "Electronic Devices", "Elettronolo", "Eleven / Gavaking",
                               "Eighting / Hudson", "Elcon", "Enerdyne Technologies, Inc.",
                               "Etona", "ESD", "ETM", "Excellent System", "Face", "Falcon", 
			       "Fujitek", "Fuuki", "Gaelco", "Game Plan, Inc.", "Game-A-Tron",
			       "GamePlan", "GameTek", "Gavaking", "Gazelle", "GCE", "GDI", "Gencomp", 
			       "Genie", "GL", "GM Shoji", "Goodhouse", "Grand Products",
			       "Graphic Techno", "Gremlin", "H.A.R. Management", "Hanafram", 
			       "Hara Industries", "Hi-max Technology Inc.", "Home Data", "Hot-B.", 
			       "Hudson", "Human", "IDSA", "IGS", "Incredible Technologies", 
			       "Interlogic", "Irem", "Irem America", "Jaleco", "Jatre", 
			       "Jetsoft", "K.K. Tokki", "KH Video", "KID", "KKI", "KKK",
			       "Kaneco", "Karateco", "Kawakusu", "Kiwako", "Komax", "Konami", 
			       "Kosuka", "Kural", "Kyugo", "Lax", "Leijac", "Leland Corp.", 
			       "Leprechaun", "Liang HWA Electronics license", "Log+Dynax", 
			       "MakeSoft", "Metro", "Midcoin", "Midway", "Mitchell", "Namco",
                               "Nazca", "Nintendo", "Nippon Amuse Co-Ltd", "NMK", 
			       "Philko Corp.", "Poby / Virus", "Psikyo", "Ramtek", "Rare", 
			       "Rock-ola", "Sammy", "Saurus", "Sega", "SemiCom", 
			       "Sigma Ent. Inc.",  "Space", "Sunsoft", "SNK", "SunA", 
			       "Sunwise", "Taito", "Taiyo", "Technos", "Technosoft",
                               "Tecmo", "T.I.C.", "Tong Electronic", "Unico", "U.S. Games",
                               "Venture Line", "Viccom", "Video System Co.", "Visco",
			       "Woodplace Inc.", "Yachiyo Electronics, Ltd", "Yumekobo", 
			       "Yun Sung"};

      for (int i = 0; i < manufacturer.length; i++) {

        roms = new TreeNode(new RomsInfo(
                                  manufacturer[i],
		                  "manufacturer/" + manufacturer[i]));

        category2.add(roms);
      }

      // Category 'Roms categories'
      TreeNode category3 = new TreeNode(
                                  utils.getString(
                                  "Tree.games.categories_label"));
				  
      top.add(category3);
      
      String[] categories = {// Adult
                             "Adult/Breakout", "Adult/Cards", "Adult/Mahjong", 
			     "Adult/Misc", "Adult/Pinball", "Adult/Platform", 
			     "Adult/Puzzle", "Adult/Tetris",
			     // Adventure
                             "Adventure", "Adventure/Fighter",
                             // Breakout
                             "Breakout",
                             // Cards
			     "Cards",
                             // Diagnostic
			     "Diagnostic",
                             // Fighter
			     "Fighter", "Fighter/Adventure", "Fighter/Character",
                             // Gambling
			     "Gambling",
			     // Gun
			     "Gun",
			     // Mahjong
			     "Mahjong", "Mahjong/Tetris",
			     // Maze
			     "Maze", "Maze/Car",
			     // Misc
			     "Misc",
			     // Pinball
			     "Pinball",
			     // Platform
			     "Platform", "Platform/Boat", "Platform/Dance", "Platform/Racing",
			     // Puzzle
			     "Puzzle", "Puzzle/Breakout", "Puzzle/Maze",
			     "Puzzle/Platform", "Puzzle/Shooter", "Puzzle/Tetris",
			     // Quiz
			     "Quiz", "Quiz/Puzzle", 
			     // Racing
			     "Racing", "Racing/Car", "Racing/F1-Car", "Racing/Horses", 
			     "Racing/Motor",
			     // Shooter
			     "Shooter", "Shooter/3D", "Shooter/Aircraft", "Shooter/Boat", 
			     "Shooter/Car", "Shooter/Character", "Shooter/Helicopter",
			     "Shooter/Misc", "Shooter/Motor", "Shooter/Space", "Shooter/Sub",
			     "Shooter/Tank", "Shooter/Western", 
			     // Sports
			     "Sports/Baseball",
			     "Sports/Basketball", "Sports/Bowling", "Sports/Boxing",
			     "Sports/DodgeBall", "Sports/Football", "Sports/Fencing",
			     "Sports/Golf", "Sports/Gymnastics",
			     "Sports/Hockey", "Sports/Horses", "Sports/HorseShoes",
			     "Sports/HorseRace", "Sports/MiniGolf", "Sports/Misc",
			     "Sports/Olympic", "Sports/PingPong", "Sports/Pool",
			     "Sports/SkateBoard", "Sports/Shuffleboard", "Sports/Skiing",
			     "Sports/Soccer", "Sports/Swimming", "Sports/Tennis",
			     "Sports/Track", "Sports/VolleyBall", "Sports/WaterSki",
			     // Wrestling
			     "Wrestling",
			     // Unknown
			     //"[Unknown]"
			     };

      for (int i = 0; i < categories.length; i++) {

        roms = new TreeNode(new RomsInfo(categories[i],
		                          "category/" + categories[i]));

        category3.add(roms);
      }
      /*
      // Category 'top rated games' [mame.dk]
      TreeNode category4 = new TreeNode(
                                  utils.getString("Tree.games.mame.dk_label"));

      top.add(category4);

      String[] topRated = {"top Adult games", "top Breakout games",
			   "top Fighter games", "top Gun games",
			   "top Mahjong games", "top Maze games",
			   "top Misc games", "top Platform games",
			   "top Puzzle games", "top Quiz games",
			   "top Racing games", "top Shooter games",
			   "top Sports games"};

      for (int i = 0; i < topRated.length; i++) {

        roms = new TreeNode(new RomsInfo(
                            topRated[i],
		            "top/" + topRated[i]));

        category4.add(roms);
      }
      */
  }

  /**
   * This method shows the list of games selected by the user.
   *
   * @param url The url representing the user selection
   *
   * @see       #showListOfGames(String)
   */
  protected void showListOfGames(URL url) {
    selection = url.toString();

    // Simulate a thread not to freeze the proggy...
    Thread thread = new Thread() {
      public void run() {
        // Show list of games
	showListOfGames(selection);
      }
    };

    // Start the thread
    thread.start();
  }

  public JPopupMenu createTreePopupMenu(JTree treeSource, int x, int y) {
    JPopupMenu treePopup = new JPopupMenu();

    createPopupMenuItem(treePopup, "Popup.tree_label", "Popup.tree_mnemonic",
                        "Popup.tree_accessible_description", "Colorset.png",
                 	new ChangeTreeColorAction(this));

    // Show the menu.
    treePopup.show(treeSource, x, y);

    return treePopup;
  }

  /**
   * This method shows the list of games selected by the user.
   *
   * @param selectedNode The name of the selected tree node
   *
   * @see   #resize(JComponent)
   * @see   com.miami.romeo.config.Utils#setProgressBarMaximum(JProgressBar, int)
   * @see   com.miami.romeo.config.Utils#setProgressBarValue(JProgressBar, int)
   * @see   com.miami.romeo.main.table.TableSorter#tableChanged(TableModelEvent)
   */
  protected void showListOfGames(final String selectedNode) {
    config = new Config(new File(System.getProperty("user.home") +
                                 File.separator + ".romeo4xmame" + 
				 File.separator + "Romeo.ini"));

    final JProgressBar progressBar = romeo.progressBar;

    if (progressBar.getValue() > 0)
      // Set progress bar value
      progressBar.setValue(0);

    // Set progress bar maximum
    // param: row lenght
    utils.setProgressBarMaximum(progressBar, row.size());

    // Prepare two status bar labels to inform the user
    romeo.statusField3.setText("");
    romeo.statusField4.setText("");

    // This vector will contain the rows
    dataRows = new Vector();

    Runnable runner = new Runnable() {
      public void run() {
        //****** Verify which list the user wanna see and show it
        if (selectedNode.endsWith("all")) {
	  for (int i = 0; i < row.size(); i++) {
	    rowLine = (Object[])row.elementAt(i);

	    available    = (String) rowLine[0];
            description  = (String) rowLine[1];
            name         = (String) rowLine[2];
            year         = (String) rowLine[3];
            manufacturer = (String) rowLine[4];
            working      = (String) rowLine[5];
            clone        = (String) rowLine[6];
            favorite     = (String) rowLine[7];
	    driver       = (String) rowLine[8];
	    category     = (String) rowLine[9];
	    sample       = (String) rowLine[10];
	    played       = (String) rowLine[11];
            path         = (String) rowLine[12];
            icon         = (String) rowLine[13];

            for (int j = 0; j < table.getColumnCount(); j++) {
              datarows = new Object[]{icon, favorite, available, description, name,
                                      year, manufacturer, working, clone, driver,
				      category, sample, played, path};
            }

            // Store the row in the vector.
	    dataRows.addElement(datarows);

	    utils.setProgressBarValue(progressBar, i);
	  }
	}
        else if (selectedNode.endsWith("favorite")) {
	  for (int i = 0; i < row.size(); i++) {
	    rowLine = (Object[])row.elementAt(i);

            favorite = (String) rowLine[7];

	    if (favorite.equals(yes)) {
	      available    = (String) rowLine[0];
              description  = (String) rowLine[1];
              name         = (String) rowLine[2];
              year         = (String) rowLine[3];
              manufacturer = (String) rowLine[4];
              working      = (String) rowLine[5];
              clone        = (String) rowLine[6];
	      driver       = (String) rowLine[8];
	      category     = (String) rowLine[9];
	      sample       = (String) rowLine[10];
	      played       = (String) rowLine[11];
              path         = (String) rowLine[12];
              icon         = (String) rowLine[13];

              for (int j = 0; j < table.getColumnCount(); j++) {
                datarows = new Object[]{icon, favorite, available, description, name,
                                        year, manufacturer, working, clone, driver,
				        category, sample, played, path};
              }

              // Store the row in the vector.
	      dataRows.addElement(datarows);

	      utils.setProgressBarValue(progressBar, i);
	    }
          }
	}
	else if (selectedNode.endsWith("available")) {
	  for (int i = 0; i < row.size(); i++) {
	    rowLine = (Object[])row.elementAt(i);

	    available = (String) rowLine[0];

	    if (available.equals(yes)) {
              description  = (String) rowLine[1];
              name         = (String) rowLine[2];
              year         = (String) rowLine[3];
              manufacturer = (String) rowLine[4];
              working      = (String) rowLine[5];
              clone        = (String) rowLine[6];
              favorite     = (String) rowLine[7];
	      driver       = (String) rowLine[8];
	      category     = (String) rowLine[9];
	      sample       = (String) rowLine[10];
	      played       = (String) rowLine[11];
              path         = (String) rowLine[12];
              icon         = (String) rowLine[13];

              for (int j = 0; j < table.getColumnCount(); j++) {
                datarows = new Object[]{icon, favorite, available, description, name,
                                        year, manufacturer, working, clone, driver,
				        category, sample, played, path};
              }

              // Store the row in the vector.
	      dataRows.addElement(datarows);

	      utils.setProgressBarValue(progressBar, i);
            }
	  }
	}
        else if (selectedNode.endsWith("original")) {
	  for (int i = 0; i < row.size(); i++) {
	    rowLine = (Object[])row.elementAt(i);

	    clone = (String) rowLine[6];

	    if (clone.equals("-")) {
	      available    = (String) rowLine[0];
              description  = (String) rowLine[1];
              name         = (String) rowLine[2];
              year         = (String) rowLine[3];
              manufacturer = (String) rowLine[4];
              working      = (String) rowLine[5];
              favorite     = (String) rowLine[7];
	      driver       = (String) rowLine[8];
	      category     = (String) rowLine[9];
	      sample       = (String) rowLine[10];
	      played       = (String) rowLine[11];
              path         = (String) rowLine[12];
              icon         = (String) rowLine[13];

              for (int j = 0; j < table.getColumnCount(); j++) {
                datarows = new Object[]{icon, favorite, available, description, name,
                                        year, manufacturer, working, clone,
					driver, category, sample, played, path};
              }

              // Store the row in the vector.
	      dataRows.addElement(datarows);

	      utils.setProgressBarValue(progressBar, i);
	    }
	  }
	}
	else if (selectedNode.endsWith("cloned")) {
	  for (int i = 0; i < row.size(); i++) {
	    rowLine = (Object[])row.elementAt(i);

	    clone = (String) rowLine[6];

	    if (!clone.equals("-")) {
	      available    = (String) rowLine[0];
              description  = (String) rowLine[1];
              name         = (String) rowLine[2];
              year         = (String) rowLine[3];
              manufacturer = (String) rowLine[4];
              working      = (String) rowLine[5];
              favorite     = (String) rowLine[7];
	      driver       = (String) rowLine[8];
	      category     = (String) rowLine[9];
	      sample       = (String) rowLine[10];
	      played       = (String) rowLine[11];
              path         = (String) rowLine[12];
              icon         = (String) rowLine[13];

              for (int j = 0; j < table.getColumnCount(); j++) {
                datarows = new Object[]{icon, favorite, available, description, name,
                                        year, manufacturer, working, clone, 
					driver, category, sample, played, path};
              }

              // Store the row in the vector.
	      dataRows.addElement(datarows);

	      utils.setProgressBarValue(progressBar, i);
	    }
	  }
	}
        else if (selectedNode.endsWith("working")) {
	  for (int i = 0; i < row.size(); i++) {
	    rowLine = (Object[])row.elementAt(i);

	    working = (String) rowLine[5];

	    if (working.equals(yes)) {
	      available    = (String) rowLine[0];
              description  = (String) rowLine[1];
              name         = (String) rowLine[2];
              year         = (String) rowLine[3];
              manufacturer = (String) rowLine[4];
              clone        = (String) rowLine[6];
              favorite     = (String) rowLine[7];
	      driver       = (String) rowLine[8];
	      category     = (String) rowLine[9];
	      sample       = (String) rowLine[10];
	      played       = (String) rowLine[11];
              path         = (String) rowLine[12];
              icon         = (String) rowLine[13];

              for (int j = 0; j < table.getColumnCount(); j++) {
                datarows = new Object[]{icon, favorite, available, description, name,
                                        year, manufacturer, working, clone, 
					driver, category, sample, played, path};
              }

              // Store the row in the vector.
	      dataRows.addElement(datarows);

	      utils.setProgressBarValue(progressBar, i);
	    }
	  }
	}
	else if (selectedNode.endsWith("trash")) {
	  for (int i = 0; i < row.size(); i++) {
	    rowLine = (Object[])row.elementAt(i);

	    working = (String) rowLine[5];

	    if (!working.equals(yes)) {
	      available    = (String) rowLine[0];
              description  = (String) rowLine[1];
              name         = (String) rowLine[2];
              year         = (String) rowLine[3];
              manufacturer = (String) rowLine[4];
              clone        = (String) rowLine[6];
              favorite     = (String) rowLine[7];
	      driver       = (String) rowLine[8];
	      category     = (String) rowLine[9];
	      sample       = (String) rowLine[10];
	      played       = (String) rowLine[11];
              path         = (String) rowLine[12];
              icon         = (String) rowLine[13];

              for (int j = 0; j < table.getColumnCount(); j++) {
                datarows = new Object[]{icon, favorite, available, description, name,
                                        year, manufacturer, working, clone,
					driver, category, sample, played, path};
              }

              // Store the row in the vector.
	      dataRows.addElement(datarows);

	      utils.setProgressBarValue(progressBar, i);
	    }
	  }
	}
	else if (selectedNode.endsWith("neogeo")) {
	  for (int i = 0; i < row.size(); i++) {
            rowLine = (Object[])row.elementAt(i);

	    driver = (String) rowLine[8];

	    if (driver.equals("neogeo")) {
	      available    = (String) rowLine[0];
              description  = (String) rowLine[1];
              name         = (String) rowLine[2];
              year         = (String) rowLine[3];
              manufacturer = (String) rowLine[4];
              working      = (String) rowLine[5];
              clone        = (String) rowLine[6];
              favorite     = (String) rowLine[7];
	      category     = (String) rowLine[9];
	      sample       = (String) rowLine[10];
	      played       = (String) rowLine[11];
              path         = (String) rowLine[12];
              icon         = (String) rowLine[13];

              for (int j = 0; j < table.getColumnCount(); j++) {
                datarows = new Object[]{icon, favorite, available, description, name,
                                        year, manufacturer, working, clone, 
					driver, category, sample, played, path};
              }

              // Store the row in the vector.
	      dataRows.addElement(datarows);

	      utils.setProgressBarValue(progressBar, i);
	    }
	  }
	}
	else if (selectedNode.endsWith("cps1")) {
	  for (int i = 0; i < row.size(); i++) {
	    rowLine = (Object[])row.elementAt(i);

	    driver = (String) rowLine[8];

	    if (driver.equals("cps1")) {
	      available    = (String) rowLine[0];
              description  = (String) rowLine[1];
              name         = (String) rowLine[2];
              year         = (String) rowLine[3];
              manufacturer = (String) rowLine[4];
              working      = (String) rowLine[5];
              clone        = (String) rowLine[6];
              favorite     = (String) rowLine[7];
	      category     = (String) rowLine[9];
	      sample       = (String) rowLine[10];
	      played       = (String) rowLine[11];
              path         = (String) rowLine[12];
              icon         = (String) rowLine[13];

              for (int j = 0; j < table.getColumnCount(); j++) {
                datarows = new Object[]{icon, favorite, available, description, name,
                                        year, manufacturer, working, clone,
					driver, category, sample, played, path};
              }

              // Store the row in the vector.
	      dataRows.addElement(datarows);

	      utils.setProgressBarValue(progressBar, i);
	    }
	  }
	}
	else if (selectedNode.endsWith("cps2")) {
	  for (int i = 0; i < row.size(); i++) {
	    rowLine = (Object[])row.elementAt(i);

	    driver = (String) rowLine[8];

	    if (driver.equals("cps2")) {
	      available    = (String) rowLine[0];
              description  = (String) rowLine[1];
              name         = (String) rowLine[2];
              year         = (String) rowLine[3];
              manufacturer = (String) rowLine[4];
              working      = (String) rowLine[5];
              clone        = (String) rowLine[6];
              favorite     = (String) rowLine[7];
	      played       = (String) rowLine[11];
              path         = (String) rowLine[12];
              icon         = (String) rowLine[13];

              for (int j = 0; j < table.getColumnCount(); j++) {
                datarows = new Object[]{icon, favorite, available, description, name,
                                        year, manufacturer, working, clone,
					driver, category, sample, played, path};
              }

              // Store the row in the vector.
	      dataRows.addElement(datarows);

	      utils.setProgressBarValue(progressBar, i);
	    }
	  }
	}
	else if (selectedNode.startsWith("file:year/")) {
	  for (int i = 0; i < row.size(); i++) {
	    rowLine = (Object[])row.elementAt(i);

	    year = (String) rowLine[3];

	    if (year.equals(selectedNode.substring(10).trim())) {
	      available    = (String) rowLine[0];
              description  = (String) rowLine[1];
              name         = (String) rowLine[2];
              manufacturer = (String) rowLine[4];
              working      = (String) rowLine[5];
              clone        = (String) rowLine[6];
              favorite     = (String) rowLine[7];
	      driver       = (String) rowLine[8];
	      category     = (String) rowLine[9];
	      sample       = (String) rowLine[10];
	      played       = (String) rowLine[11];
              path         = (String) rowLine[12];
              icon         = (String) rowLine[13];

              for (int j = 0; j < table.getColumnCount(); j++) {
                datarows = new Object[]{icon, favorite, available, description, name,
                                        year, manufacturer, working, clone,
					driver, category, sample, played, path};
              }

              // Store the row in the vector.
	      dataRows.addElement(datarows);

	      utils.setProgressBarValue(progressBar, i);
	    }
	  }
	}
	else if (selectedNode.startsWith("file:manufacturer/")) {
	  for (int i = 0; i < row.size(); i++) {
	    rowLine = (Object[])row.elementAt(i);

	    manufacturer = (String) rowLine[4];

	    if (manufacturer.equals(selectedNode.substring(18).trim()) ||
                manufacturer.startsWith("[" +
		                          selectedNode.substring(18).trim())) {
	      available    = (String) rowLine[0];
              description  = (String) rowLine[1];
              name         = (String) rowLine[2];
	      manufacturer = (String) rowLine[4];
              year         = (String) rowLine[3];
              working      = (String) rowLine[5];
              clone        = (String) rowLine[6];
              favorite     = (String) rowLine[7];
              driver       = (String) rowLine[8];
	      played       = (String) rowLine[11];
              path         = (String) rowLine[12];
              icon         = (String) rowLine[13];

              for (int j = 0; j < table.getColumnCount(); j++) {
                datarows = new Object[]{icon, favorite, available, description, name,
                                        year, manufacturer, working, clone,
					driver, category, sample, played, path};
              }

              // Store the row in the vector.
	      dataRows.addElement(datarows);

	      utils.setProgressBarValue(progressBar, i);
	    }
	  }
	}
	else if (selectedNode.startsWith("file:category/")) {
	  for (int i = 0; i < row.size(); i++) {
            rowLine = (Object[])row.elementAt(i);

            category = (String) rowLine[9];

	    if (category.equalsIgnoreCase(selectedNode.substring(14).trim())) {
	      available    = (String) rowLine[0];
              description  = (String) rowLine[1];
              name         = (String) rowLine[2];
	      manufacturer = (String) rowLine[4];
              year         = (String) rowLine[3];
              working      = (String) rowLine[5];
              clone        = (String) rowLine[6];
              favorite     = (String) rowLine[7];
              driver       = (String) rowLine[8];
	      sample       = (String) rowLine[10];
	      played       = (String) rowLine[11];
              path         = (String) rowLine[12];
              icon         = (String) rowLine[13];

              for (int j = 0; j < table.getColumnCount(); j++) {
                datarows = new Object[]{icon, favorite, available, description, name,
                                        year, manufacturer, working, clone,
					driver, category, sample, played, path};
              }

              // Store the row in the vector.
	      dataRows.addElement(datarows);

	      utils.setProgressBarValue(progressBar, i);
	    }
	  }
	}
        /*
	else if (selectedNode.startsWith("file:top/")) {
	  final String[] topRated = getTopRatedGamesList(
	                              selectedNode.substring(9).trim());

	  for (int z = 0; z < topRated.length; z++) {
            for (int i = 0; i < row.size(); i++) {
	      rowLine = (Object[])row.elementAt(i);

	      description = (String) rowLine[1];

	      if (description.equals(topRated[z])) {
                available    = (String) rowLine[0];
                name         = (String) rowLine[2];
                year         = (String) rowLine[3];
                manufacturer = (String) rowLine[4];
                working      = (String) rowLine[5];
                clone        = (String) rowLine[6];
                favorite     = (String) rowLine[7];
		driver       = (String) rowLine[8];
		category     = (String) rowLine[9];
		sample       = (String) rowLine[10];
		played       = (String) rowLine[11];

	        for (int j = 0; j < table.getColumnCount(); j++) {
	          datarows = new Object[]{favorite, available, description, name,
	                                  year, manufacturer, working, clone,
					  driver, category, sample, played};
                }

                // Store the row in the vector.
                dataRows.addElement(datarows);

		utils.setProgressBarValue(progressBar, i);
	      }
	    }
          }
	}
        */

	// Tell the sorter there' s a new model.
        sorter.tableChanged(null);

	int spacing = config.getInt("Table.intercell_spacing");
	int rowHeight = config.getInt("Table.row_height");

	table2.setIntercellSpacing(new Dimension(spacing, spacing));
	table2.setRowHeight(rowHeight);
	table2.repaint();

	//setCellSize();
        table.setIntercellSpacing(new Dimension(spacing, spacing));
	table.setRowHeight(rowHeight);
	table.repaint();
	
	Dimension frameDim = frame.getSize();
        int frameWidth = frameDim.width;
	int dividerLocation = horzSplitPane.getDividerLocation();
        int inWidth = frameWidth - dividerLocation - 280;
	Dimension tableSize = table.getPreferredSize();
        int inHeight = (int)tableSize.getHeight();
        innerPort.setPreferredSize(new Dimension(inWidth, inHeight));

	table3.setIntercellSpacing(new Dimension(spacing, spacing));
	table3.setRowHeight(rowHeight);
        table3.repaint();

        // Update status bar message
	if (config.getBoolean("View.statusbar")) {
	  if (selectedNode.endsWith("all"))
	    message = " " + utils.getString(
                      "roms.supported_all") + " " +
                      table.getRowCount();
	  else if (selectedNode.endsWith("favorite"))
	    message = " " + utils.getString(
                      "roms.supported_favorite") + " " +
                      table.getRowCount();
          else if (selectedNode.endsWith("available"))
	    message = " " + utils.getString(
                      "roms.supported_available") + " " +
                      table.getRowCount();
	  else if (selectedNode.endsWith("original"))
	    message = " " + utils.getString(
                      "roms.supported_original") + " " +
                      table.getRowCount();
          else if (selectedNode.endsWith("cloned"))
	    message = " " + utils.getString(
                      "roms.supported_cloned") + " " +
                      table.getRowCount();
	  else if (selectedNode.endsWith("working"))
	    message = " " + utils.getString(
                      "roms.supported_working") + " " +
                      table.getRowCount();
	  else if (selectedNode.endsWith("trash"))
	    message = " " + utils.getString(
                      "roms.supported_nonworking") + " " +
                      table.getRowCount();
	  else if (selectedNode.endsWith("neogeo") ||
	           selectedNode.endsWith("cps1") || 
		   selectedNode.endsWith("cps2") ||
		   selectedNode.startsWith("file:category/"))
	    message = " " + utils.getString(
                      "roms.supported_neogeo") + " " +
                      table.getRowCount();
	  else if (selectedNode.startsWith("file:year/"))
	    message = " " + utils.getString(
                      "roms.supported_peryear") +
                      " " + selectedNode.substring(10).trim() + ": " +
		      table.getRowCount();
	  else if (selectedNode.startsWith("file:manufacturer/"))
	    message = " " + utils.getString(
                      "roms.supported_manufacturer") +
                      " " + selectedNode.substring(18).trim() + ": " +
		      table.getRowCount();
          /*
	  else if (selectedNode.startsWith("file:top/"))
	    message = " " + utils.getString(
	              "roms.supported_top_rated") +
                      " " + table.getRowCount();
         */

	  romeo.statusField2.setText(message);
	}
      }
    };
    SwingUtilities.invokeLater(runner);
  }

  /**
   * [mame.dk] is an arcade rom and information repository. This site
   * is a 'must' for every arcade fan. It contains a subsection with
   * the lists of the best games rated by the users.
   * This method retrieves the list of top rated games selected by the
   * from [mame.dk].
   *
   * @param selection The name of the list of games to retrieve from [mame.dk]
   *
   * @return The <code>list</code> of top rated games
   */
   /*
  protected String[] getTopRatedGamesList(String selection) {
    ArrayList topRatedGamesList = new ArrayList();

    try {
      URL url = new URL("http://www.mame.dk/top.phtml");

      URLConnection con = url.openConnection();

      LineNumberReader lnr = new LineNumberReader(
                                   new InputStreamReader(con.getInputStream()));

      String line;

      while ((line = lnr.readLine()) != null) {
        if (line.startsWith(selection)) {
          int lineNumber = lnr.getLineNumber();
          lnr.mark(lineNumber + 1);
          while ((line = lnr.readLine()) != null) {
	    if (line.startsWith("</TD> <TD valign=")) {
	      // Parse line
	      int x = line.indexOf("/\">");
	      line = line.substring(x + 3);
	      int y = line.indexOf("</a>");
	      line = line.substring(0, y);
              topRatedGamesList.add(line);
	    }
	    if (line.equals("</TD></TR></TABLE></TD></TR></TABLE>"))
	      break;
	  }
	}
      }

      lnr.close();
    }
    catch(java.net.MalformedURLException mue) {
      System.err.println(mue.getMessage());
    }
    catch(IOException ioe) {
      System.err.println(ioe.getMessage());
    }

    return (String[])topRatedGamesList.toArray(new String[0]);
  }
  */
  
  /**
   * This method resizes the tables.
   *
   * @param comp The component to resize
   *
   * @since 0.1.3pre4
   */
  public void resize(JComponent comp) {
    // Get frame size
    Dimension frameDim = frame.getSize();
    int frameWidth = frameDim.width;
    int frameHeight = frameDim.height;
    // Get horzSplitPane location
    int horzLocation = horzSplitPane.getDividerLocation();
    // This is the new innerPort and headerPort2 width:
    // frame size - horzSplitPane location - (headerPort1 + headerPort3 size)
    int inWidth = frameWidth - horzLocation - 280;
    // Get table height
    Dimension tableSize = table.getPreferredSize();
    int inHeight = (int)tableSize.getHeight();
    Dimension headPortSize = headerPort2.getPreferredSize();
    int outWidth = (int)headPortSize.getWidth();
    int outHeight = (int)headPortSize.getHeight();
    headerPort2.setPreferredSize(new Dimension(inWidth, outHeight));
    innerPort.setPreferredSize(new Dimension(inWidth, inHeight));
    vertSplitPane.setDividerLocation(frameHeight - (frameHeight / 3));
    SwingUtilities.updateComponentTreeUI(comp);
    
    // Call Config to save the configuration file
    config = new Config(new File(System.getProperty("user.home") +
                                 File.separator + ".romeo4xmame" +
				 File.separator + "Romeo.ini"));

    config.add("horz_split_location", horzSplitPane.getDividerLocation());
    config.save();
  }

  // ***** End of tree stuff

  // *******************************************************
  // ********************   Tables   ***********************
  // *******************************************************

  /**
   * This method creates a panel containing table options.
   *
   * @return <code>JPanel</code>
   */
  public JPanel createTableOptions() {
    config = new Config(new File(System.getProperty("user.home") + 
                                 File.separator + ".romeo4xmame" +
				 File.separator + "Romeo.ini"));

    JPanel tableOptions = new JPanel();
    tableOptions.setLayout(new BorderLayout());
    int panelWidth = (int)tableOptions.getWidth();
    tableOptions.setPreferredSize(new Dimension(panelWidth, 55));
    tableOptions.setBorder(new EmptyBorder(5 , 10, 5, 10));

    JPanel spacing = new JPanel();
    spacing.setLayout(new BorderLayout());

    JPanel spacing1 = new JPanel();
    spacing1.setLayout(new BorderLayout());

    // Get inter-cell spacing value
    int interCellSpacingValue = config.getInt("Table.intercell_spacing");

    interCellSpacingLabel = new JLabel(utils.getString(
                     "Table.intercell_spacing") + ": " + interCellSpacingValue);
    spacing1.add(interCellSpacingLabel, BorderLayout.NORTH);

    // Slider to set inter-cell spacing
    interCellSpacingSlider = new JSlider(JSlider.HORIZONTAL, 0, 10,
                                         interCellSpacingValue);
    Dimension sDim = interCellSpacingSlider.getPreferredSize();
    int interCellSpacingSliderWidth = (int)sDim.getWidth();
    interCellSpacingSlider.setPreferredSize(new Dimension(interCellSpacingSliderWidth, 
                                                          18));
    interCellSpacingSlider.getAccessibleContext().setAccessibleName(
                                    utils.getString("Table.intercell_spacing"));
    
    interCellSpacingLabel.setLabelFor(interCellSpacingSlider);
    interCellSpacingSlider.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
	int spacing = ((JSlider)e.getSource()).getValue();
	interCellSpacingLabel.setText(utils.getString(
                                            "Table.intercell_spacing") + ": " + spacing);
        table2.setIntercellSpacing(new Dimension(spacing, spacing));
        table2.repaint();
        table.setIntercellSpacing(new Dimension(spacing, spacing));
        table.repaint();
	table3.setIntercellSpacing(new Dimension(spacing, spacing));
        table3.repaint();

	if (romeo.settings != null && romeo.settings.isVisible())
	  romeo.settings.interCellSpacingSlider.setValue(spacing);
      }
    });
    interCellSpacingSlider.addFocusListener(new FocusAdapter() {
      public void focusLost(FocusEvent e) {
        JSlider slider = (JSlider)e.getSource();
        int value = slider.getValue();
	config = new Config(new File(System.getProperty("user.home") + 
	                             File.separator + ".romeo4xmame" + 
				     File.separator + "Romeo.ini"));
        config.add("Table.intercell_spacing", value);
	config.save();
      }
    });
    spacing1.add(interCellSpacingSlider, BorderLayout.CENTER);

    spacing.add(spacing1, BorderLayout.WEST);

    JPanel spacing2 = new JPanel();
    spacing2.setLayout(new BorderLayout());

    // Get row height value
    int rowHeightValue = config.getInt("Table.row_height");

    rowHeightLabel = new JLabel(utils.getString("Table.row_height") +
                                ": " + rowHeightValue);
    spacing2.add(rowHeightLabel, BorderLayout.NORTH);

    // Slider to set row height
    rowHeightSlider = new JSlider(JSlider.HORIZONTAL, 20, 100, rowHeightValue);
    sDim = rowHeightSlider.getPreferredSize();
    int rowHeightSliderWidth = (int)sDim.getWidth();
    rowHeightSlider.setPreferredSize(new Dimension(rowHeightSliderWidth, 18));
    rowHeightSlider.getAccessibleContext().setAccessibleName(
                                                   utils.getString("Table.row_height"));
    rowHeightLabel.setLabelFor(rowHeightSlider);
    rowHeightSlider.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
	int height = ((JSlider)e.getSource()).getValue();
	rowHeightLabel.setText(utils.getString("Table.row_height") + ": " + height);
	table2.setRowHeight(height);
        table2.repaint();
        table.setRowHeight(height);
        table.repaint();
	Dimension frameDim = frame.getSize();
        int frameWidth = frameDim.width;
	int dividerLocation = horzSplitPane.getDividerLocation();
        int inWidth = frameWidth - dividerLocation - 280;
	Dimension tableSize = table.getPreferredSize();
        int inHeight = (int)tableSize.getHeight();
        innerPort.setPreferredSize(new Dimension(inWidth, inHeight));
	table3.setRowHeight(height);
        table3.repaint();

        outerPane.getVerticalScrollBar().setUnitIncrement(height);

	if (romeo.settings != null && romeo.settings.isVisible())
	  romeo.settings.rowHeightSlider.setValue(height);
      }
    });
    rowHeightSlider.addFocusListener(new FocusAdapter() {
      public void focusLost(FocusEvent e) {
        JSlider slider = (JSlider)e.getSource();
        int value = slider.getValue();
	config = new Config(new File(System.getProperty("user.home") + 
	                             File.separator + ".romeo4xmame" + 
				     File.separator + "Romeo.ini"));
        config.add("Table.row_height", value);
	config.save();
      }
    });
    spacing2.add(rowHeightSlider, BorderLayout.CENTER);

    spacing.add(spacing2, BorderLayout.EAST);

    tableOptions.add(spacing, BorderLayout.WEST);

    JPanel lines = new JPanel();
    lines.setLayout(new BorderLayout());

    // Component to set horizontal lines in the tables
    showHorizontalLinesCheckBox = new JCheckBox(
                                         utils.getString("Table.horz_lines"),
                                         config.getBoolean("Table.horz_lines"));
    showHorizontalLinesCheckBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	boolean flag = ((JCheckBox)e.getSource()).isSelected();
	table2.setShowHorizontalLines(flag);
        table2.repaint();
        table.setShowHorizontalLines(flag);
        table.repaint();
	table3.setShowHorizontalLines(flag);
        table3.repaint();
	config = new Config(new File(System.getProperty("user.home") +
	                             File.separator + ".romeo4xmame" +
				     File.separator + "Romeo.ini"));
	config.add("Table.horz_lines", flag);
	config.save();

        if (romeo.settings != null && romeo.settings.isVisible())
	  romeo.settings.showHorizontalLinesCheckBox.setSelected(flag);
      }
    });
    lines.add(showHorizontalLinesCheckBox, BorderLayout.NORTH);

    // Component to set vertical lines in the tables
    showVerticalLinesCheckBox = new JCheckBox(utils.getString(
                                              "Table.vert_lines"),
                                              config.getBoolean(
					      "Table.vert_lines"));
    showVerticalLinesCheckBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	boolean flag = ((JCheckBox)e.getSource()).isSelected();
	table2.setShowVerticalLines(flag);
        table2.repaint();
        table.setShowVerticalLines(flag);
        table.repaint();
	table3.setShowVerticalLines(flag);
        table3.repaint();
	config = new Config(new File(System.getProperty("user.home") + 
	                             File.separator + ".romeo4xmame" + 
				     File.separator + "Romeo.ini"));
	config.add("Table.vert_lines", flag);
	config.save();

	if (romeo.settings != null && romeo.settings.isVisible())
	  romeo.settings.showVerticalLinesCheckBox.setSelected(flag);
      }
    });
    lines.add(showVerticalLinesCheckBox, BorderLayout.SOUTH);

    tableOptions.add(lines, BorderLayout.CENTER);

    // Panel containing buttons to set external tables background color
    JPanel colors = new JPanel();
    colors.setLayout(new BorderLayout());
    
    JLabel label = new JLabel(utils.getString("Table.background_colors"));
    colors.add(label, BorderLayout.NORTH);

    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new BorderLayout());

    // Button to set background color for the internal table
    intTablesColor = new JButton();
    cs = new ColorSwatch(new Color(config.getInt("Table.internal_red"),
                                   config.getInt("Table.internal_green"),
				   config.getInt("Table.internal_blue")));
    intTablesColor.setIcon(cs);
    intTablesColor.setToolTipText(utils.getString(
                                  "Table.internal_background_tooltip"));

    ActionListener l = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (table.getRowCount() > 0) {
          Color current = new Color(127, 255, 212);

	  Component comp = columnModel2.getColumn(
	                   0).getCellRenderer().getTableCellRendererComponent(
	                   table2, table2.getValueAt(0, 0), false, false, 0, 0);

	  current = comp.getBackground();

	  // Bring up a color chooser
	  Color c = JColorChooser.showDialog(
		                             romeo,
	                                     utils.getString(
					     "ColorChooser.title"),
		                             current
	                                    );
					    
          int r = c.getRed();
	  int g = c.getGreen();
	  int b = c.getBlue();

	  config = new Config(new File(System.getProperty("user.home") +
	                               File.separator + ".romeo4xmame" +
				       File.separator + "Romeo.ini"));
	  config.add("Table.internal_red", r);
	  config.add("Table.internal_green", g);
	  config.add("Table.internal_blue", b);
	  config.save();

          cs = new ColorSwatch(c);
	  intTablesColor.setIcon(cs);
          renderer = new TableRenderer(c);
	  
	  String header = utils.getString("Table.column_played");
	  
	  iRenderer = createIndicatorCellRenderer();

	  for (int i = 0; i < table.getColumnCount(); i++) {
	    if (!table.getColumnName(i).equals(header))
	      columnModel1.getColumn(i).setCellRenderer(renderer);
	    else
              columnModel1.getColumn(i).setCellRenderer(iRenderer);
	  }
	  table.repaint();

	  if (romeo.settings != null && romeo.settings.isVisible())
            romeo.settings.intTablesColor.setIcon(cs);
	}
      }
    };

    intTablesColor.addActionListener(l);
    
    buttonPanel.add(intTablesColor, BorderLayout.WEST);
    buttonPanel.setBorder(new EmptyBorder(1, 1, 1, 1));

    // Button to set background color for the external tables
    extTablesColor = new JButton();
    cs = new ColorSwatch(new Color(config.getInt("Table.external_red"),
                                   config.getInt("Table.external_green"),
				   config.getInt("Table.external_blue")));
    extTablesColor.setIcon(cs);
    extTablesColor.setToolTipText(utils.getString(
                                  "Table.external_background_tooltip"));

    l = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (table.getRowCount() > 0) {
          Color current = new Color(127, 255, 212);

	  Component comp = columnModel2.getColumn(
	                   0).getCellRenderer().getTableCellRendererComponent(
	                   table2, table2.getValueAt(0, 0), false, false, 0, 0);
	  current = comp.getBackground();

	  // Bring up a color chooser
	  Color c = JColorChooser.showDialog(
		                             romeo,
	                                     utils.getString(
					     "ColorChooser.title"),
		                             current
	                                    );
					    
          int r = c.getRed();
	  int g = c.getGreen();
	  int b = c.getBlue();

	  config = new Config(new File(System.getProperty("user.home") +
	                               File.separator + ".romeo4xmame" +
				       File.separator + "Romeo.ini"));
	  config.add("Table.external_red", r);
	  config.add("Table.external_green", g);
	  config.add("Table.external_blue", b);
	  config.save();

          cs = new ColorSwatch(c);
	  extTablesColor.setIcon(cs);
          renderer = new TableRenderer(c);
	  columnModel2.getColumn(0).setCellRenderer(renderer);
	  table2.repaint();
	  columnModel3.getColumn(0).setCellRenderer(renderer);
	  table3.repaint();

	  if (romeo.settings != null && romeo.settings.isVisible())
            romeo.settings.intTablesColor.setIcon(cs);
	}
      }
    };

    extTablesColor.addActionListener(l);

    buttonPanel.add(extTablesColor, BorderLayout.EAST);
    colors.add(buttonPanel, BorderLayout.SOUTH);

    tableOptions.add(colors, BorderLayout.EAST);

    return tableOptions;
  }

  /**
   * This method creates three tables and then adds them to a
   * a JScrollPane. Table 1 and table 3 will be fixed, (only 
   * vert. scroll), table 2 will be scrollable horizontally and
   * vertically. Table 2 will be scrollable behind table 1 and 
   * table 3.
   *
   * @return <code>JScrollPane</code>
   *
   * @see    #createTableCellRenderer(Color)
   * @see    #createIndicatorCellRenderer()
   * @see    com.miami.romeo.main.table.TableSorter#addMouseListenerToHeaderInTable(JTable, SortButtonRenderer)
   * @see    com.miami.romeo.main.Romeo#setLocked(boolean, Container)
   * @see    #setComponentsLocked(boolean)
   * @see    com.miami.romeo.main.Options#runMame()
   * @see    #createPopupMenu(JTable, int, int, int, int)
   *
   * @since  0.1b
   */
  public JScrollPane createTables() {
    config = new Config(new File(System.getProperty("user.home") + 
                                 File.separator + ".romeo4xmame" + 
				 File.separator + "Romeo.ini"));

    headerPanel = new JPanel();

    // Columns names
    columnNames = new Vector();
    columnNames.addElement(utils.getString("Table.column_icon"));
    columnNames.addElement(utils.getString("Table.column_favorite"));
    columnNames.addElement(utils.getString("Table.column_availability"));
    columnNames.addElement(utils.getString("Table.column_description"));
    columnNames.addElement(utils.getString("Table.column_name"));
    columnNames.addElement(utils.getString("Table.column_year"));
    columnNames.addElement(utils.getString("Table.column_manufacturer"));
    columnNames.addElement(utils.getString("Table.column_working"));
    columnNames.addElement(utils.getString("Table.column_cloneof"));
    columnNames.addElement("Driver");
    columnNames.addElement(utils.getString("Table.column_category"));
    columnNames.addElement("Audio Sample");
    columnNames.addElement(utils.getString("Table.column_played"));
    columnNames.addElement(utils.getString("Table.column_path"));
    
    // Model for tables
    model = new TableModel();
    
    // This class triggers a table sort when a column heading is clicked 
    // in the JTables.
    sorter = new TableSorter(model);

    // Renderer for tables headers
    sbr = new SortButtonRenderer();

    // Column model for the main table
    columnModel1 = new DefaultTableColumnModel();
    
    // Internal table background color
    Color background = new Color(config.getInt("Table.internal_red"),
                                 config.getInt("Table.internal_green"),
			         config.getInt("Table.internal_blue"));

    // Renderer for table cells with text or images
    renderer = createTableCellRenderer(background);

    // Renderer for table cells with a progress bar
    iRenderer = createIndicatorCellRenderer();

    TableColumn col = null;

    // Create the columns and add them to the column model
    for (int i = 0; i < columnNames.size(); i++) {
      col = new TableColumn(i);
      col.setHeaderValue(columnNames.elementAt(i).toString());
      col.setHeaderRenderer(sbr);
      if (i == 12)
        // The column 11 needs a different renderer as its cells
	// contain progress bars
	col.setCellRenderer(iRenderer);
      else
        col.setCellRenderer(renderer);
      columnModel1.addColumn(col);
    }

    // Create the main internal table to contain roms info.
    table = new JTable(sorter, columnModel1);
    // Add a mouse listener to the Table to trigger a table sort
    // when a column heading is clicked in the table.
    sorter.addMouseListenerToHeaderInTable(table, sbr);
    int spacing = config.getInt("Table.intercell_spacing");
    table.setShowHorizontalLines(config.getBoolean("Table.horz_lines"));
    table.setShowVerticalLines(config.getBoolean("Table.vert_lines"));
    table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
    table.getTableHeader().setReorderingAllowed(false);
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    int in = (int)((table.getPreferredSize()).getHeight());

    // List selection model
    ListSelectionModel rowSM = table.getSelectionModel();

    /* List Selection Listener for the tables */
    ListSelectionListener ls = new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        //Ignore extra messages.
        if (e.getValueIsAdjusting()) return;

	ListSelectionModel lsm = (ListSelectionModel)e.getSource();

        config = new Config(new File(System.getProperty("user.home") +
	                             File.separator + ".romeo4xmame" +
				     File.separator + "Romeo.ini"));

        if (lsm.isSelectionEmpty()) {
          //no rows are selected
          setComponentsLocked(false);
          romeo.setLocked(true, romeo.options);
        }
	else {
	  //selectedRow is selected
          int selectedRow = lsm.getMinSelectionIndex();

	  // Programmatically highlight the same rows in the other tables
	  table2.setRowSelectionInterval(selectedRow, selectedRow);
	  table3.setRowSelectionInterval(selectedRow, selectedRow);

	  setComponentsLocked(true);

          // Set any info about the selected game: to review...
	  gameName = table2.getValueAt(selectedRow, 0).toString();
	  cloneOf  = table.getValueAt(selectedRow, 6).toString();
	  driver   = table.getValueAt(selectedRow, 7).toString();
	  sample   = table.getValueAt(selectedRow, 9).toString();

          // Enable any components
	  if (gameIsRunnable(gameName, cloneOf, driver)) {
	    romeo.getJMenuBar().getMenu(0).getItem(0).setEnabled(true);
            romeo.getToolBar().getComponentAtIndex(0).setEnabled(true);
            romeo.setLocked(false, romeo.options);
            romeo.options.setOptionsComponentsLocked();
          }
          else {
	    romeo.getJMenuBar().getMenu(0).getItem(0).setEnabled(false);
            romeo.getToolBar().getComponentAtIndex(0).setEnabled(false);
            romeo.setLocked(true, romeo.options);
          }

          // Add new action listeners to the internet components: to review...
	  for (int i = 0; i < 7; i++) {
            ActionListener[] actions = romeo.getJMenuBar().getMenu(4).getItem(
                                                       i).getActionListeners();
            for (int j = 0; j < actions.length; j++) {
              romeo.getJMenuBar().getMenu(4).getItem(i).removeActionListener(
                                                                   actions[j]);
            }
          }

          romeo.getJMenuBar().getMenu(4).getItem(0).addActionListener(
                       new DownloadAction(Games.this, "rom", gameName));
          romeo.getJMenuBar().getMenu(4).getItem(1).addActionListener(
                       new DownloadAction(Games.this, "snap", gameName));
          romeo.getJMenuBar().getMenu(4).getItem(2).addActionListener(
                       new DownloadAction(Games.this, "flyer", gameName));
          romeo.getJMenuBar().getMenu(4).getItem(3).addActionListener(
                       new DownloadAction(Games.this, "marquee", gameName));
          romeo.getJMenuBar().getMenu(4).getItem(4).addActionListener(
                       new DownloadAction(Games.this, "cabinet", gameName));
	  romeo.getJMenuBar().getMenu(4).getItem(5).addActionListener(
                       new DownloadAction(Games.this, "sample", gameName));
          romeo.getJMenuBar().getMenu(4).getItem(6).addActionListener(
                       new DownloadAction(Games.this, "artwork", gameName));

	  // Update the status bar messages
	  if (config.getBoolean("View.statusbar")) {
	    romeo.statusField3.setText(" " +
	                               utils.getString("Table.column_name") +
                                       ": " + gameName);

            if (cloneOf.equals("-"))
              romeo.statusField4.setText(" " + utils.getString(
                                         "statusField.original"));
            else
              romeo.statusField4.setText(" " + utils.getString(
                                         "statusField.cloneof") +
                                         " " + cloneOf);
	  }
        }
      }
    };

    rowSM.addListSelectionListener(ls);

    // Set cell size
    setCellSize();

    // Create two new column model
    columnModel2 = new DefaultTableColumnModel();
    columnModel3 = new DefaultTableColumnModel();

    // Get the external tables backgroud color
    background = new Color(config.getInt("Table.external_red"),
                           config.getInt("Table.external_green"),
			   config.getInt("Table.external_blue"));
					    
    // Create a new renderer
    renderer = createTableCellRenderer(background);

    // Remove some columns from the main table and add them
    // to the external tables
    col = columnModel1.getColumn(4);
    columnModel1.removeColumn(col);
    columnModel2.addColumn(col);
    col.setCellRenderer(renderer);
    col = columnModel1.getColumn(2);
    columnModel1.removeColumn(col);
    columnModel3.addColumn(col);
    col.setCellRenderer(renderer);

    // External fixed table 2 contaiing games names
    table2 = new JTable(sorter, columnModel2);
    sorter.addMouseListenerToHeaderInTable(table2, sbr);
    table2.getTableHeader().setReorderingAllowed(false);
    table2.setIntercellSpacing(new Dimension(spacing, spacing));
    table2.setShowHorizontalLines(config.getBoolean("Table.horz_lines"));
    table2.setShowVerticalLines(config.getBoolean("Table.vert_lines"));
    table2.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    
    /* List Selection Listener for the table 2 */
    ListSelectionListener ls2 = new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        //Ignore extra messages.
        if (e.getValueIsAdjusting()) return;

	ListSelectionModel lsm = (ListSelectionModel)e.getSource();

        if (lsm.isSelectionEmpty()) {
          //no rows are selected
        }
	else {
          int selectedRow = lsm.getMinSelectionIndex(); //selectedRow is selected

	  table.setRowSelectionInterval(selectedRow, selectedRow);

        }
      }
    };

    ListSelectionModel rowSM2 = table2.getSelectionModel();
    rowSM2.addListSelectionListener(ls2);

    // Table 3
    table3 = new JTable(sorter, columnModel3);
    sorter.addMouseListenerToHeaderInTable(table3, sbr);
    table3.getTableHeader().setReorderingAllowed(false);
    table3.setIntercellSpacing(new Dimension(spacing, spacing));
    table3.setShowHorizontalLines(config.getBoolean("Table.horz_lines"));
    table3.setShowVerticalLines(config.getBoolean("Table.vert_lines"));
    table3.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

    rowSM2 = table3.getSelectionModel();
    rowSM2.addListSelectionListener(ls2);

    // Add a mouse listener to act on tables row right-click.
    MouseListener ml = new MouseListener() {
      // Needed by this interface
      public void mouseEntered (MouseEvent e) {
        // to handle
      }
      
      // Needed by this interface
      public void mousePressed(MouseEvent e) {
        // to handle
      }

      /* A double click on a selected row starts the game if it's playable */
      public void mouseClicked(MouseEvent e) {
        JTable tableSource = (JTable)e.getSource();

        int row = tableSource.rowAtPoint(e.getPoint());

        if (romeo.getToolBar().getComponentAtIndex(0).isEnabled() &&
	   (e.getClickCount() == 2))
	  romeo.options.runMame();
      }

      /* Right click shows a popup menu */
      public void mouseReleased(MouseEvent e) {
        JTable tableSource = (JTable)e.getSource();

        if (e.isShiftDown()|| e.isControlDown()|| e.isAltDown()) {
          return;
        }
        if (e.isMetaDown()) {
          int x = e.getX();
          int y = e.getY();

          int row    = tableSource.rowAtPoint(e.getPoint());
          int column = tableSource.columnAtPoint(e.getPoint());

          if (/*tableSource == table && */tableSource.isRowSelected(row))
            createPopupMenu(tableSource, x, y, row, column);
	    
	    if (e.isPopupTrigger()) {
               popup.show(e.getComponent(), e.getX(), e.getY());
	    }
        }
      }

      // Needed by this interface
      public void mouseExited(MouseEvent e) {
        // to handle
      }
    };

    table2.addMouseListener(ml);
    table.addMouseListener(ml);
    table3.addMouseListener(ml);

    JScrollPane sp = new JScrollPane(
                                   new JViewport(),
                                   JScrollPane.VERTICAL_SCROLLBAR_NEVER,
                                   JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    headScroll = sp.getHorizontalScrollBar();
    headerPort1 = new JViewport();
    headerPort2 = new JViewport();
    headerPort3 = new JViewport();
    headerPort1.add(table2.getTableHeader());
    headerPort2.add(table.getTableHeader());
    headerPort3.add(table3.getTableHeader());
    headerPanel.setLayout(new BorderLayout());
    headerPanel.add(headScroll, "South");
    JPanel lowerHeadPanel = new JPanel();
    lowerHeadPanel.add(headerPort1);
    lowerHeadPanel.add(headerPort2);
    lowerHeadPanel.add(headerPort3);
    headerPanel.add(lowerHeadPanel, "North");
    Dimension d = new Dimension(150, in);
    table.setPreferredScrollableViewportSize(d);
    table2.setPreferredScrollableViewportSize(d);
    table3.setPreferredScrollableViewportSize(d);
    innerPort = new JViewport();
    innerPort.add(table);
    Dimension portSize0 = innerPort.getPreferredSize();
    int inHeight = (int)portSize0.getHeight();
    // New
    int frameWidth = config.getInt("Romeo.SIZE_X");
    int dividerLocation = horzSplitPane.getDividerLocation();
    int inWidth = frameWidth - dividerLocation - 280;
    Dimension headPortSize = headerPort2.getPreferredSize();
    int outHeight = (int)headPortSize.getHeight();
    int outWidth = inWidth;
    headerPort2.setPreferredSize(new Dimension(outWidth, outHeight));
    innerPort.setPreferredSize(new Dimension(inWidth, inHeight));
    sp.setViewport(innerPort);
    //sp.getViewport().setBackground(java.awt.Color.red);
    headScroll.getModel().addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        Point q = headerPort2.getViewPosition();
        Point p = innerPort.getViewPosition();
        int val = headScroll.getModel().getValue();
        p.x = val;
        q.x = val;
        headerPort2.setViewPosition(p);
        headerPort2.repaint(innerPort.getViewRect());
        innerPort.setViewPosition(p);
        innerPort.repaint(innerPort.getViewRect());
      }
    });

    tables = new Vector();
    tables.addElement(table2);
    tables.addElement(innerPort);
    tables.addElement(table3);

    scrollPanel = new JPanel();
    scrollPanel.add((JTable)tables.elementAt(0));
    scrollPanel.add((JViewport)tables.elementAt(1));
    scrollPanel.add((JTable)tables.elementAt(2));
    outerPane = new JScrollPane(scrollPanel,
                                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    // the following code sets the unit increment for vertical scrolling to n pixels
    outerPane.getVerticalScrollBar().setUnitIncrement(config.getInt("Table.row_height"));
    outerPane.setColumnHeaderView(headerPanel);
    outerPort = new JViewport();
    outerPort.add(scrollPanel);
    Dimension portSize = outerPort.getPreferredSize();
    int portWidth = (int)portSize.getWidth();
    outerPort.setMinimumSize(new Dimension(portWidth, 492));
    outerPort.setPreferredSize(new Dimension(portWidth, 492));
    outerPane.setViewport(outerPort);
    //outerPane.getViewport().setBackground(java.awt.Color.red);

    return outerPane;
  }

  /**
   * Table Model.
   *
   * @since 0.1a
   */
  class TableModel extends AbstractTableModel {
    // Return columnes number.
    public int getColumnCount() {
      return columnNames.size();
    }

    // Return rows number.
    public int getRowCount() {
      if (dataRows == null)
        return 0;
      else
        return dataRows.size();
    }

    // Return the value at the position row, column.
    public Object getValueAt(int row, int column) {
      return ((Object[]) (dataRows.elementAt(row)))[column];
    }

    // Return a name for the column.
    public String getColumnName(int column) {
      return (String)columnNames.elementAt(column);
    }

    // Set a value at the position row, column.
    public void setValueAt(Object aValue, int row, int column) {
      Object[] element = (Object[])dataRows.elementAt(row);
      element[column] = aValue;
      dataRows.setElementAt(element, row);

      // Notify all listeners that the value of the cell at
      // (row, column) has been updated
      //fireTableCellUpdated(row, column);
    }

    /**
     * JTables use this method to determine the default renderer/
     * editor for each cell.  For example, if we didn't implement
     * this method, then the first column (favorite games) would
     * contain text (path_to_image_icon), rather than an image.
     */
    public Class getColumnClass(int c) {
      return Object.class;
    }
  }
  
  /**
   * This method returns a new DefaultTableCellRenderer to render table
   * cells containing a label.
   *
   * @param background The background color to pass to the table renderer
   *
   * @return <code>TableRenderer</code>
   */
  public TableRenderer createTableCellRenderer(Color background) {
    // Renderer for tables cells with text or images
    renderer = new TableRenderer(background);

    return renderer;
  }

  /**
   * This method returns a renderer for the table cells
   * containing a progress bar.
   *
   * @return <code>IndicatorCellRenderer</code>
   *
   * @see    com.miami.romeo.main.table.IndicatorCellRenderer#setStringPainted(boolean)
   * @see    com.miami.romeo.main.table.IndicatorCellRenderer#setBackground(Color)
   * @see    com.miami.romeo.main.table.IndicatorCellRenderer#setLimits(Hashtable)
   */
  public IndicatorCellRenderer createIndicatorCellRenderer() {
    config = new Config(new File(System.getProperty("user.home") +
                                 File.separator + ".romeo4xmame" +
				 File.separator + "Romeo.ini"));

    int maxValue;
    String xmamePath = getXmamePath();

    if (!xmamePath.equals("<NOT SET>"))
      maxValue = config.getInt(getNameOfList(xmamePath) + ".total_played");
    else
      maxValue = 0;

    Color background = new Color(config.getInt("Table.internal_red"),
                                 config.getInt("Table.internal_green"),
			         config.getInt("Table.internal_blue"));

    // Renderer for table cells with a progress bar
    if (maxValue > 0)
      iRenderer = new IndicatorCellRenderer(0, maxValue);
    else
      iRenderer = new IndicatorCellRenderer();
    iRenderer.setStringPainted(true);
    iRenderer.setBackground(background);

    // Set limit value and fill color
    Hashtable limitColors = new Hashtable();
    limitColors.put(new Integer( 0), Color.green);
    limitColors.put(new Integer(60), Color.yellow);
    limitColors.put(new Integer(80), Color.red);
    iRenderer.setLimits(limitColors);

    return iRenderer;
  }

  /**
   * This method sets table cell height and width.
   */
  public void setCellSize() {
    config = new Config(new File(System.getProperty("user.home") +
                                 File.separator + ".romeo4xmame" +
				 File.separator + "Romeo.ini"));

    table.setRowHeight(config.getInt("Table.row_height"));

    TableColumn column = null;

    for (int i = 0; i < table.getColumnCount(); i++) {
      column = columnModel1.getColumn(i);
      if (i == 3) {
        column.setPreferredWidth(400); // description column is bigger
      }
      else if (i == 5) {
        column.setPreferredWidth(50); // year column
      }
      else if (i == 6) {
        column.setPreferredWidth(300); // manufacturer column
      }
      else if (i == 8) {
        column.setPreferredWidth(200); // cloneof column
      }
      else if (i == 10) {
        column.setPreferredWidth(150); // category
      }
      else if (i == 11) {
        column.setPreferredWidth(150); // Audio sample
      }
      else if (i == 13) {
        column.setPreferredWidth(300); // rom path
      }
      else {
        column.setPreferredWidth(100); // Other columns
      }
    }
  }

  /**
   * This method locks/unlocks some menu items.
   *
   * boolean locked If <code>true</code> enable these menu items;
   *                else disable them.
   */
  public void setComponentsLocked(boolean locked) {
    romeo.getJMenuBar().getMenu(4).getItem(0).setEnabled(locked);
    romeo.getJMenuBar().getMenu(4).getItem(1).setEnabled(locked);
    romeo.getJMenuBar().getMenu(4).getItem(2).setEnabled(locked);
    romeo.getJMenuBar().getMenu(4).getItem(3).setEnabled(locked);
    romeo.getJMenuBar().getMenu(4).getItem(4).setEnabled(locked);
    romeo.getJMenuBar().getMenu(4).getItem(5).setEnabled(locked);
    romeo.getJMenuBar().getMenu(4).getItem(6).setEnabled(locked);
  }
  
  // *******************************************************
  // ******************* Popup menu  ***********************
  // *******************************************************

  /**
   * This method creates a popup menu for the tables.
   *
   * @param tableSource  The component invoker
   * @param x      The x coordinate in invoker's coordinate space
   *               at which the popup menu is to be displayed
   * @param y      The y coordinate in invoker's coordinate space
   *               at which the popup menu is to be displayed
   * @param row    The table row at the position x,y in the
   *               coordinate space of the component invoker
   * @param column The table column at the position x,y in the
   *               coordinate space of the component invoker
   *
   * @return       <code>JPopupMenu</code>
   *
   * @see          #createPopupMenuItem(JPopupMenu, String, String, String, String, Action)
   * @see          #createPopupCheckBoxMenuItem(JPopupMenu, String, String, String, String, Action, int)
   * @see          #createPopupSubMenuItem(JMenu, String, String, String, String, Action)
   *
   * @since        0.1a
   */
  public JPopupMenu createPopupMenu(JTable tableSource, int x, int y,
                                    int row, int column) {
    // ***** Create popup menu
    popup = new JPopupMenu();

    // ***** create menu items
    // Item to run an available game.
    if (romeo.getToolBar().getComponentAtIndex(0).isEnabled())
      createPopupMenuItem(popup, "Popup.run_label", "Popup.run_mnemonic",
                          "Popup.run_accessible_description", "Exec.png",
                 	  new RunGameAction(this));
    else
      createPopupMenuItem(popup, "Popup.run_label", "Popup.run_mnemonic",
                          "Popup.run_accessible_description", "Exec.png",
         		  null);

    popup.addSeparator();

    // Item to set/unset a game as favorite.
    createPopupCheckBoxMenuItem(popup, "Popup.preferite_label",
                                "Popup.preferite_mnemonic",
                                "Popup.preferite_accessible_description", 
				"Bookmark.png",
                                new SetAsFavoriteAction(this,
                                gameName,
                                table.getValueAt(row, 1).toString(),
                                row), row);

    popup.addSeparator();

    // Item to verify an available rom.
    if (table3.getValueAt(row, 0).toString().equals("Yes"))
      createPopupMenuItem(popup, "Popup.verify_label", "Popup.verify_mnemonic",
                          "Popup.verify_accessible_description", "Unlock.png",
                          new VerifyRomAction(this, gameName));
    else
      createPopupMenuItem(popup, "Popup.verify_label", "Popup.verify_mnemonic",
                          "Popup.verify_accessible_description", "Lock.png",
			  null);

    popup.addSeparator();

    //****** Add a menu with submenus to the popupmenu
    infoMenu = new JMenu("Info");
    infoMenu.setIcon(utils.createImageIcon16x16("Info.png"));

    // Item to list number of colors used for a game
    String OS = System.getProperty("os.name").toLowerCase();
    //System.out.println(OS);
    if (OS.indexOf("windows") > -1)
      createPopupSubMenuItem(infoMenu, "Popup.listcolors_label",
                             "Popup.listcolors_mnemonic",
                             "Popup.listcolors_accessible_description",
			     "Colorset.png",
			     null);
    else
      createPopupSubMenuItem(infoMenu, "Popup.listcolors_label",
                             "Popup.listcolors_mnemonic",
                             "Popup.listcolors_accessible_description",
			     "Colorset.png",
			     new ListColorsAction(this, gameName));
			
    // Item to list clones of a game
    createPopupSubMenuItem(infoMenu, "Popup.listclones_label",
                           "Popup.listclones_mnemonic",
                           "Popup.listclones_accessible_description",
		           "Info.png",
			   new ListClonesAction(this, gameName, cloneOf));
			
    // Item to list used audio samples
    createPopupSubMenuItem(infoMenu, "Popup.listsamples_label",
                           "Popup.listsamples_mnemonic",
                           "Popup.listsamples_accessible_description",
	                   "Info.png",
			   new ListSamplesAction(this, gameName, cloneOf));
			
    infoMenu.addSeparator();

    //List cpu usage statistics per year
    createPopupSubMenuItem(infoMenu, "Popup.listcpu_label",
                        "Popup.listcpu_mnemonic",
     	                "Popup.listcpu_accessible_description",
			"Info.png",
			new ListCpuAction(this, gameName, cloneOf));

    popup.add(infoMenu);

    popup.addSeparator();

    // Item to download roms
    createPopupMenuItem(popup, "Popup.download_rom_label",
                        "Popup.download_rom_mnemonic",
                        "Popup.download_rom_accessible_description",
			"Rom.png",
	        	new DownloadAction(this, "rom", gameName));

    // Item to download a snapshots
    createPopupMenuItem(popup, "Popup.download_snap_label",
                        "Popup.download_snap_mnemonic",
 	                "Popup.download_snap_accessible_description",
			"Image.png",
			new DownloadAction(this, "snap", gameName));

    // Item to download flyers
    createPopupMenuItem(popup, "Popup.download_flyer_label",
                        "Popup.download_flyer_mnemonic",
	                "Popup.download_flyer_accessible_description",
			"Image.png",
			new DownloadAction(this, "flyer", gameName));

    // Item to download marquees
    createPopupMenuItem(popup, "Popup.download_marquee_label",
                        "Popup.download_marquee_mnemonic",
	                "Popup.download_marquee_accessible_description",
			"Image.png",
			new DownloadAction(this, "marquee", gameName));

    // Item to download cabinets
    createPopupMenuItem(popup, "Popup.download_cabinet_label",
                        "Popup.download_cabinet_mnemonic",
		        "Popup.download_cabinet_accessible_description",
			"Image.png",
			new DownloadAction(this, "cabinet", gameName));
			
    // Item to download audio samples
    createPopupMenuItem(popup, "Popup.download_sample_label",
                        "Popup.download_sample_mnemonic",
		        "Popup.download_sample_accessible_description",
			"Tgz.png",
			new DownloadAction(this, "sample", gameName));
			 
    // Item to download artworks
    createPopupMenuItem(popup, "Popup.download_artwork_label",
                        "Popup.download_artwork_mnemonic",
		        "Popup.download_artwork_accessible_description",
			"Tgz.png",
			new DownloadAction(this, "artwork", gameName));

    // Show the menu.
    popup.show(tableSource, x, y);

    return popup;
  }

  /**
   * This method creates a generic JMenuItem for the popup menu.
   *
   * @param popup                 The popup menu to add the menu item to
   * @param label                 The menu item' s text
   * @param mnemonic              The keyboard mnemonic on the current component
   * @param accessibleDescription The accessible description of this object
   * @param filename              The icon this component will display
   * @param action                The Action for the ActionEvent source
   *
   * @return                      <code>JMenuItem</code> for the popup menu
   */
  public JMenuItem createPopupMenuItem(JPopupMenu popup, String label,
                                       String mnemonic,
		                       String accessibleDescription,
				       String filename,
				       Action action) {

    JMenuItem mi = (JMenuItem) popup.add(new JMenuItem(
                                                      utils.getString(label)));
    mi.setMnemonic(utils.getMnemonic(mnemonic));
    mi.getAccessibleContext().setAccessibleDescription(
                                       utils.getString(accessibleDescription));
    mi.setIcon(utils.createImageIcon16x16(filename));
    mi.addActionListener(action);
    if(action == null) {
      mi.setEnabled(false);
    }

    return mi;
  }
  
  /**
   * This method creates a generic sub JMenuItem for the popup menu.
   *
   * @param menu                  The menu to add the sub menu item to
   * @param label                 The menu item' s text
   * @param mnemonic              The keyboard mnemonic on the current component
   * @param accessibleDescription The accessible description of this object
   * @param filename              The icon this component will display
   * @param action                The Action for the ActionEvent source
   *
   * @return                      <code>JMenuItem</code> for the popup menu
   */
  public JMenuItem createPopupSubMenuItem(JMenu menu, String label,
                                          String mnemonic,
		                          String accessibleDescription,
					  String filename, Action action) {

    JMenuItem mi = (JMenuItem) menu.add(new JMenuItem(utils.getString(label)));
    mi.setMnemonic(utils.getMnemonic(mnemonic));
    mi.getAccessibleContext().setAccessibleDescription(utils.getString(
                                                       accessibleDescription));
    mi.setIcon(utils.createImageIcon16x16(filename));
    mi.addActionListener(action);
    if(action == null) {
      mi.setEnabled(false);
    }

    return mi;
  }

  /**
   * This method creates a JCheckBoxMenuItem for the popup menu to set a
   * given game as a favorite.
   *
   * @param popup                 The popup menu to add the JCheckBoxMenuItem to
   * @param label                 The components' s text
   * @param mnemonic              The keyboard mnemonic on the current component
   * @param accessibleDescription The accessible description of this object
   * @param filename              The icon this component will display
   * @param action                The Action for the ActionEvent source
   * @param row                   The table row where to know if the component
   *                              is enabled
   *
   * @return                      <code>JCheckBoxMenuItem</code> for the popup menu
   */
  public JCheckBoxMenuItem createPopupCheckBoxMenuItem(
                                JPopupMenu popup, String label,
                                String mnemonic, String accessibleDescription,
                                String filename, Action action, int row) {

    JCheckBoxMenuItem mi = (JCheckBoxMenuItem) popup.add(
                                                     new JCheckBoxMenuItem(
                                                     utils.getString(label)));
    mi.setMnemonic(utils.getMnemonic(mnemonic));
    mi.getAccessibleContext().setAccessibleDescription(
                                      utils.getString(accessibleDescription));
    mi.setIcon(utils.createImageIcon16x16(filename));
    mi.addActionListener(action);
    if(action == null) {
      mi.setEnabled(false);
    }
    if (table.getValueAt(row, 1).toString().equals("Yes")) {
      mi.setSelected(true);
    }

    return mi;
  }

  /**
   * This method returns the table popup menu.
   *
   * @return <code>JPopupMenu</code>
   */
  public JPopupMenu getPopupMenu() {
    return popup;
  }

  // *******************************************************
  // ********************   Actions  ***********************
  // *******************************************************

  /**
   * This abstract action change tree' s background color.
   *
   * @param games The existing class to add this interface to
   */
  class ChangeTreeColorAction extends AbstractAction {
    Games games;
    protected ChangeTreeColorAction(Games games) {
      super("ChangeTreeColor");
      this.games = games;
    }

    public void actionPerformed(ActionEvent e) {
      Color current = tree.getBackground();

      // Bring up a color chooser
      Color c = JColorChooser.showDialog(games,
	                                 utils.getString(
					 "ColorChooser.title"),
		                         current
	                                 );

      tree.setBackground(c);
      treeRenderer = new TreeNodeRenderer(c);
      tree.setCellRenderer(treeRenderer);

      int r = c.getRed();
      int g = c.getGreen();
      int b = c.getBlue();

      config = new Config(new File(System.getProperty("user.home") +
	                           File.separator + ".romeo4xmame" +
	 		           File.separator + "Romeo.ini"));
      config.add("Tree.background_red", r);
      config.add("Tree.background_green", g);
      config.add("Tree.background_blue", b);
      config.save();
    }
  }

  /**
   * This abstract action runs a given game.
   *
   * @param games The existing class to add this interface to
   */
  class RunGameAction extends AbstractAction {
    Games games;
    protected RunGameAction(Games games) {
      super("RunGame");
      this.games = games;
    }

    public void actionPerformed(ActionEvent e) {
      Thread thread = new Thread() {
        public void run() {
          Runnable runner = new Runnable() {
            public void run() {
              getPopupMenu().setVisible(false);
              romeo.options.runMame();
            }
          };
          SwingUtilities.invokeLater(runner);
        }
      };
      thread.start();
    }
  }

  /**
   * This abstract action sets a given game as favorite/non-favorite.
   *
   * @param Games    The class Games
   * @param gameName The game name
   * @param favorite "Yes" or "No"
   * @param tableRow Table row containing data to change
   *
   * @see            com.miami.romeo.main.DataIO#updateListOfGames(Vector, String, String)
   */
  class SetAsFavoriteAction extends AbstractAction {
    Games games;
    String gameName;
    String favorite;
    int tableRow;

    protected SetAsFavoriteAction(Games games, String gameName,
                                  String favorite, int tableRow) {
      super("SetAsFavorite");
      this.games = games;
      this.gameName = gameName;
      this.favorite = favorite;
      this.tableRow = tableRow;
    }

    public void actionPerformed(ActionEvent e) {
      String f;
      String xmamePath = getXmamePath();
      String listName = getNameOfList(xmamePath);

      // Call this class to save favorite status in a specific text file
      config = new Config(new File(System.getProperty("user.home") +
                                   File.separator + ".romeo4xmame" +
				   File.separator + "favorites" +
				   listName.substring(4) + ".ini"));

      // Selected game is not more a favorite
      if (favorite.equals("Yes"))
        f = "No";
      else
        f = "Yes";

      config.add(gameName, f);
      config.save();

      for (int i = 0; i < row.size(); i++) {
        Object[] line = (Object[])row.elementAt(i);
        if (line[2].equals(gameName)) {
          line[7] = f;
	  row.setElementAt(line, i);
	  break;
	}
      }
      
      table.setValueAt(f, tableRow, 1);
      table.repaint();

      dataio = new DataIO(romeo);
      dataio.updateListOfGames(row, xmamePath, listName);
    }
  }

  /**
   * This abstract action verifies a rom.
   *
   * @param games    The existing class to add this interface to
   * @param gameName The game name
   */
  class VerifyRomAction extends AbstractAction {
    Games games;
    String gameName;
    protected VerifyRomAction(Games games, String gameName) {
      super("VerifyRom");
      this.games = games;
      this.gameName = gameName;
    }

    public void actionPerformed(ActionEvent e) {
      // Class to read and write configuration file
      config = new Config(new File(System.getProperty("user.home") +
                                   File.separator + ".romeo4xmame" +
				   File.separator + "Romeo.ini"));
      String[] romDir = getRomDirs();
      String romPath  = getGamePath(romDir, gameName);

      if (romPath.length() > 0) {
        String verifyromInfo;

        xmame = new Xmame(romeo);

        if (xmame.romIsCorrect(getXmamePath(), romPath, gameName))
          verifyromInfo = gameName + " " +
                          utils.getString("Popup.verify_messagebox_correct") +
			  ".";
        else
          verifyromInfo = gameName + " " +
                          utils.getString("Popup.verify_messagebox_incorrect") +
			  ".";

        JOptionPane.showMessageDialog(null, verifyromInfo,
                                      utils.getString("Messagebox.info_title"),
                                      JOptionPane.INFORMATION_MESSAGE);
      }
    }
  }

  /**
   * This abstract action shows the number of colors used by a
   * given game.
   *
   * @param games    The existing class to add this interface to
   * @param gameName The game name
   *
   * @see            com.miami.romeo.xmame.Xmame#getRomColors(String, String)
   */
  class ListColorsAction extends AbstractAction {
    Games games;
    String gameName;
    protected ListColorsAction(Games games, String gameName) {
      super("ListColors");
      this.games = games;
      this.gameName = gameName;
    }

    public void actionPerformed(ActionEvent e) {
      xmame = new Xmame(romeo);
      String colors = xmame.getRomColors(getXmamePath(), gameName);

      String colors_info = gameName + " " +
                           utils.getString("Popup.listcolors_messagebox_use") +
                           " " + colors + " " +
                           utils.getString(
			   "Popup.listcolors_messagebox_colors") + ".";

      JOptionPane.showMessageDialog(null, colors_info,
                                    utils.getString("Messagebox.info_title"),
                                    JOptionPane.INFORMATION_MESSAGE);
    }
  }
  
  /**
   * This abstract action shows the list of clones of a given game.
   *
   * @param games    The existing class to add this interface to
   * @param gameName The game name
   * @param cloneOf  The original version of this game
   *
   * @see            com.miami.romeo.xmame.Xmame#getClonesOfGame(String, String)
   */
  class ListClonesAction extends AbstractAction {
    Games games;
    String gameName;
    String cloneOf;
    protected ListClonesAction(Games games, String gameName, String cloneOf) {
      super("ListClones");
      this.games    = games;
      this.gameName = gameName;
      this.cloneOf  = cloneOf;
    }

    public void actionPerformed(ActionEvent e) {
      xmame = new Xmame(romeo);
      String[] clones = xmame.getClonesOfGame(getXmamePath(), gameName);

      if (!config.get("Language").equals("English")) {
        String newFirstLine = clones[0].replaceAll("Name:",
                                                   utils.getString(
						   "Table.column_name") + ":");
        newFirstLine = clones[0].replaceAll("Clone of:",
                                            utils.getString(
					    "Table.column_cloneof") + ":");
					    
        clones[0] = newFirstLine;
      }

      createInfoDialog(utils.getString("InfoBox.title_clones") +
                       " " + gameName, gameName, cloneOf, clones);
    }
  }

  /**
   *  This abstract action shows the list of samples used by a given game.
   *
   * @param games    The existing class to add this interface to
   * @param gameName The game name
   * @param cloneOf  The original version of a given game
   *
   * @see            com.miami.romeo.xmame.Xmame#getGameSamples(String, String)
   */
  class ListSamplesAction extends AbstractAction {
    Games games;
    String gameName;
    String cloneOf;
    protected ListSamplesAction(Games games, String gameName, String cloneOf) {
      super("ListSamples");
      this.games    = games;
      this.gameName = gameName;
      this.cloneOf  = cloneOf;
    }

    public void actionPerformed(ActionEvent e) {
      xmame = new Xmame(romeo);
      String[] samples = xmame.getGameSamples(getXmamePath(), gameName);

      if (samples.length > 0) {
        createInfoDialog(utils.getString("InfoBox.title_samples"),
	                 gameName, cloneOf, samples);
      }
      else {
        String[] error = {utils.getString("InfoBox.no_samples")};
        createInfoDialog(utils.getString("InfoBox.title_samples"),
	                 gameName, cloneOf, error);
      }
    }
  }

  /**
   * This abstract action lists cpu statistics per year.
   *
   * @param games    The existing class to add this interface to
   * @param gameName The game name
   * @param cloneOf  The original version of a given game
   *
   * @see            com.miami.romeo.xmame.Xmame#listCpuUsage(String)
   * @see            #createInfoDialog(String, String, String, String[])
   */
  class ListCpuAction extends AbstractAction {
    Games games;
    String gameName;
    String cloneOf;
    protected ListCpuAction(Games games, String gameName, String cloneOf) {
      super("ListCpu");
      this.games = games;
      this.gameName = gameName;
      this.cloneOf  = cloneOf;
    }

    public void actionPerformed(ActionEvent e) {
      xmame = new Xmame(romeo);
      String[] usage = xmame.listCpuUsage(getXmamePath());

      usage[0] = "----\t8\t16\t32";

      createInfoDialog(utils.getString("CpuUsage.title"), gameName, cloneOf,
                       usage);
    }
  }

  /**
   * This abstract action downloads files (ROMs, snaps, flyers etc.) from 
   * the internet.
   *
   * @param games         The existing class to add this interface to
   * @param fileType      The file type (ROM, snap, flyer, etc.)
   * @param fileName      The file name
   * @param destination   The local directory where to store file in
   * @param urlPath       The file URL
   * @param fileExtension The file extension
   * @param server        The file search server
   */
  class DownloadAction extends AbstractAction {
    Games games;
    String fileType, fileName, destination, urlPath, fileExtension, server;

    protected DownloadAction(Games games, String fileType, String fileName) {
      super("Download");
      this.games = games;
      this.fileType = fileType;
      this.fileName = fileName;
    }

    public void actionPerformed(ActionEvent e) {
      String config_name = System.getProperty("user.home") + File.separator + 
                                              ".romeo4xmame" + File.separator + 
					      "Romeo.ini";
      File config_file = new File(config_name);
      config = new Config(config_file);

      if (fileType.equals("rom")) {
        String paths = config.get("xmame.rom_dir");
        String[] dir = utils.parseValues(paths, ';');
        destination = dir[0];
        urlPath = parseURL(config.get("xmame.rom_server"), fileName);
      }
      else if (fileType.equals("snap")) {
        destination = config.get("xmame.snap_dir");
	urlPath = parseURL(config.get("xmame.snap_server"), fileName);
      }
      else if (fileType.equals("cabinet")) {
        destination = config.get("xmame.cabinet_dir");
	urlPath = parseURL(config.get("xmame.cabinet_server"), fileName);
      }
      else if (fileType.equals("flyer")) {
        destination = config.get("xmame.flyer_dir");
	urlPath = parseURL(config.get("xmame.flyer_server"), fileName);
      }
      else if (fileType.equals("marquee")) {
        destination = config.get("xmame.marquee_dir");
        urlPath = parseURL(config.get("xmame.marquee_server"), fileName);
      }
      else if (fileType.equals("sample")) {
	destination = config.get("xmame.sample_dir");
        urlPath = parseURL(config.get("xmame.sample_server"), fileName);
      }
      else if (fileType.equals("artwork")) {
	destination = config.get("xmame.artwork_dir");
        urlPath = parseURL(config.get("xmame.artwork_server"), fileName);
      }

      if ((!destination.equals("")) && (!urlPath.equals(""))) {
	// Parse url
        int idx = urlPath.lastIndexOf(fileName + ".");
        server = urlPath.substring(0, idx);
	idx = urlPath.lastIndexOf('.');
        fileExtension = urlPath.substring(idx);

        // Call Downloader window
	downloader = new Downloader(server, destination, fileName + fileExtension);
      }
      else {
        JOptionPane.showMessageDialog(null,
	                              utils.getString(
				      "Messagebox.software_not_configured"),
                                      utils.getString("Messagebox.error_title"),
                                      JOptionPane.ERROR_MESSAGE);
      }
    }
  } // ***** End of tables stuff

  // *******************************************************
  // ******************* message area *********************
  // *******************************************************

  /**
   * This method creates a scrollable message area to display
   * any errors or diagnostics from subprocesses.
   *
   * @return <code>JScrollPane</code>
   *
   * @see    #appendMessage(String, String)
   */
  public JScrollPane createMessageArea() {
    // Create a new JTextPane
    messageArea = new JTextPane();
    messageArea.setEditable(false);
    appendMessage("First Line", "This message area will display " +
                  "any errors or diagnostics from subprocesses:");

    // Get user name
    String userName = config.get("User.name");
    if (userName.equals(""))
      userName = System.getProperty("user.name");
    // Create a second list item
    appendMessage("Info", "Romeo 0.1.3pre4 loaded by user " + userName +
                  " +++++++++++++++++++++++++++");

    scrollMessages = new JScrollPane(messageArea);

    return scrollMessages;
  }
  
  // *******************************************************
  // ****************** Utility Methods ********************
  // *******************************************************

  /**
   * This method shows the list of all games on startup.
   */
  public void initGames() {
    checkXmame(getXmamePath());
  }

  /**
   * This method performs any operations if a new xmame executable 
   * is selected.
   *
   * @param xmamePath The path to the xmame executable
   *
   * @see   #checkListOfGames(String)
   * @see   #appendMessage(String, String)
   */
  public void checkXmame(String xmamePath) {
    if (!xmamePath.equals("<NOT SET>")) {
      File f = new File(xmamePath);

      // Check if selected xmame exists
      if (f.exists())
	// Check if a list of games has been created for the selected
	// executable
	checkListOfGames(xmamePath);
    }
    else {
      if (row != null && dataRows != null) {
        row.clear();
        dataRows.clear();
        // Tell the sorter there' s a new model.
        sorter.tableChanged(null);

	table2.repaint();
	table.repaint();
        table3.repaint();

	Dimension frameDim = frame.getSize();
        int frameWidth = frameDim.width;
	int dividerLocation = horzSplitPane.getDividerLocation();
        int inWidth = frameWidth - dividerLocation - 280;
	Dimension tableSize = table.getPreferredSize();
        int inHeight = (int)tableSize.getHeight();
        innerPort.setPreferredSize(new Dimension(inWidth, inHeight));
      }

      // Disable all enabled components
      setComponentsLocked(false);
      romeo.setLocked(true, romeo.options);

      // Disable the selection tree
      if (tree != null && tree.isEnabled()) {
        tree.setEnabled(false);
	appendMessage("Error", "Selection tree disabled.");
      }

      updateStatusField(xmamePath);
      romeo.statusField2.setText("");
      romeo.statusField3.setText("");
      romeo.statusField4.setText("");
    }
  }

  /**
   * This method checks wheter a list of games has been created for a
   * given xmame executable.
   *
   * @param xmamePath The path to the xmame executable
   *
   * @see   #saveListOfGames()
   * @see   com.miami.romeo.main.DataIO#listOfGamesExists(String, String)
   * @see   com.miami.romeo.main.DataIO#saveListOfGames(String, String)
   * @see   com.miami.romeo.main.DataIO#getListOfGames(String, String)
   * @see   #appendMessage(String, String)
   */
  public void checkListOfGames(String xmamePath) {
    // Call Config to read the configuration file
    String configName = System.getProperty("user.home") +
	                File.separator + ".romeo4xmame" +
	                File.separator + "Romeo.ini";
    File configFile = new File(configName);
    config = new Config(configFile);

    int oldRoms = config.getInt("xmame.available_roms");

    dataio = new DataIO(romeo);

    int newRoms = dataio.countRoms(xmamePath);

    boolean listExists = dataio.listOfGamesExists(xmamePath, getNameOfList(xmamePath));

    // List doesn' t exist...
    if (!listExists || (oldRoms != newRoms)) {
      // Disable the selection tree
      if (tree != null && tree.isEnabled()) {
        tree.setEnabled(false);
	appendMessage("Error", "Selection tree disabled.");
      }

      appendMessage("Error", "No list of games found!");
      int result = JOptionPane.showConfirmDialog(null,
                                                 utils.getString("OptionPane.new_xmame"));

      // Save a new list of games
      if(result == JOptionPane.YES_OPTION) {
        if (updateDialog  == null || !updateDialog.isVisible()) {
	  saveListOfGames();
        }
      }
      else if(result == JOptionPane.NO_OPTION) {
        JOptionPane.showMessageDialog(null, utils.getString(
                                      "Messagebox.list_notupgraded"),
                                      utils.getString("Messagebox.info_title"),
                                      JOptionPane.INFORMATION_MESSAGE);
      }
    }
    else {
      tree.setSelectionPath(null);

      // Load the list of games into a memory stack to
      // guarantee a faster access to the data
      row = dataio.getListOfGames(xmamePath, getNameOfList(xmamePath));

      // Inform the user about the list loaded
      appendMessage("Info", "Loaded a list of games with " +
                    row.size() + " entries.");

      if (row.size() > 0)
        updateRomeo(xmamePath);
    }
  }

  /**
   * This method creates a box to show the updating of the list
   * of games.
   *
   * @see com.miami.romeo.main.DataIO#saveListOfGames(String, String)
   * @see #updateRomeo(String)
   */
  public void saveListOfGames() {
    // Let' s use a thread not to freeze Romeo
    Thread thread = new Thread() {
      public void run() {
        String xmamePath = getXmamePath();

        File xmameFile = new File(xmamePath);

        if (xmameFile.exists()) {
          // Disable the selection tree
          if (tree != null && tree.isEnabled()) {
            tree.setEnabled(false);
	    appendMessage("Error", "Selection tree disabled.");
          }

          if (row != null && dataRows != null) {
            row.clear();
            dataRows.clear();
            // Tell the sorter there' s a new model.
            sorter.tableChanged(null);
            innerPort.setPreferredSize(new Dimension(
                              (int)innerPort.getPreferredSize().getWidth(), 0));
          }

          try {
            //Call the class that handles the I/O operations
            dataio = new DataIO(romeo);
            // Save the list of games
            row = dataio.saveListOfGames(getXmamePath(), getNameOfList(xmamePath));
          }
          catch (InterruptedException ie) {
            System.err.println(ie.getMessage());
          }

          // Inform the user about the list
          appendMessage("Info", "Loaded a list of games with " +
                        row.size() + " entries.");

          if (row.size() > 0)
            updateRomeo(xmamePath);
        }
        else {
          System.out.println("xmame executable not found");
        }
      }
    };
    thread.start();
  }

  /**
   * This method updates some components if the list of games has been loaded.
   *
   * @see #updateStatusField(String)
   * @see #enableTree()
   * @see #checkCatver()
   * @see #highlightAllGamesNode()
   * @see #setXmameSoundPlugins(String)
   */
  public void updateRomeo(String xmamePath) {
    // Retrieve the xmame version and show it in the status bar
    updateStatusField(xmamePath);

    // Enable the selection tree
    enableTree();

    tree.setSelectionPath(null);

    // Check if catver.ini exists
    checkCatver();

    // Highlight all games node
    highlightAllGamesNode();

    // Update digital sound and mixer plugins supported by the
    // selected xmame
    setXmameSoundPlugins(xmamePath);
  }

  /**
   * This method shows xmame version in the status bar.
   *
   * @param xmamePath The path to the xmame executable
   *
   * @see   #getXmameVersion(String)
   */
  public void updateStatusField(String xmamePath) {
    String xmameVersion = getXmameVersion(xmamePath);

    romeo.statusField.setText(" " + xmameVersion);
  }
  
  /**
   * This method returns xmame version.
   *
   * @param  xmamePath The path to the xmame executable
   *
   * @return <code>String</code>
   *
   * @see    com.miami.romeo.xmame.Xmame#getVersion(String)
   */
  public String getXmameVersion(String xmamePath) {
    xmame = new Xmame(romeo);

    String version = xmame.getVersion(xmamePath);
    
    return version;
  }
  
  /**
   * This method enables the selection tree.
   *
   * @see #appendMessage(String, String)
   */
  public void enableTree() {
    if (tree != null && !tree.isEnabled()) {
      tree.setEnabled(true);
      appendMessage("Info", "Selection tree enabled.");
    }
  }
  
  /**
   * This method enables/disables tree categories nodes.
   *
   * @param tree    The tree to enable/disable
   * @param enabled if <code>true</code> enable the tree; else
   *                disable it.
   *
   * @see   com.miami.romeo.main.tree.TreeNode#getChildCount()
   */
  public void setCategoryNodeEnabled(JTree tree, boolean enabled) {
    // Get tree model
    DefaultTreeModel treeModel = (DefaultTreeModel)tree.getModel();
    // Get root node
    TreeNode rootNode = (TreeNode)treeModel.getRoot();
    // Get wanted enumeration
    Enumeration e = rootNode.postorderEnumeration();
    TreeNode node = null;
    Object nodeInfo = null;
    String title = null;
    while (e.hasMoreElements()) { // all nodes
      node = (TreeNode)e.nextElement(); // Get Node
      nodeInfo = node.getUserObject();
      title = nodeInfo.toString();
      if (title.startsWith("Adult") || title.startsWith("Adventure") ||
          title.startsWith("Breakout") || title.startsWith("Cards") ||
	  title.startsWith("Diagnostic") || title.startsWith("Fighter") ||
          title.startsWith("Gambling") || title.startsWith("Gun") ||
	  title.startsWith("Mahjong") || title.startsWith("Maze") ||
	  title.startsWith("Misc") || title.startsWith("Pinball") ||
	  title.startsWith("Platform") ||
	  title.startsWith("Puzzle") || title.startsWith("Quiz") ||
	  title.startsWith("Racing") || title.startsWith("Shooter") ||
	  title.startsWith("Sports") || title.equals("Wrestling") ||
	  title.equals("[Unknown]")) {
	int beforeChildCount = node.getChildCount();
        node.setEnabled(enabled);
        int afterChildCount  = node.getChildCount();
        if (beforeChildCount == afterChildCount) {
          treeModel.nodeChanged(node);
        } else {
          treeModel.reload();
        }
      }
    }
  }

  /**
   * This method checks if catver.ini exists: if the file exists it enables
   * any specific tree nodes.
   *
   * @see #setCategoryNodeEnabled(JTree, boolean)
   * @see #appendMessage(String, String)
   */
  public void checkCatver() {
    if (tree.isEnabled()) {
      // Path to configuration file
      String config_name = System.getProperty("user.home") + File.separator +
                           ".romeo4xmame" + File.separator + "Romeo.ini";
      // Configuration file
      File config_file = new File(config_name);
      // Class to read and write configuration file
      config = new Config(config_file);
      // Get the path to catver.ini
      String catverPath = config.get("xmame.catver.ini_path");
      
      if (catverPath.equals("")) {
        // Disable category nodes
	setCategoryNodeEnabled(tree, false);
	if (messageArea != null)
	  appendMessage("Error", "Path to catver.ini not set..." +
	                "Category nodes disabled.");
      }
      else {
        File catver = new File(catverPath);
        if (!catver.exists()) {
	  setCategoryNodeEnabled(tree, false);
	  if (messageArea != null)
	    appendMessage("Error", "File catver.ini not found..." +
	                  "Category nodes disabled.");
        }
        else {
          setCategoryNodeEnabled(tree, true);
	  if (messageArea != null)
	    appendMessage("Info", "Category nodes enabled.");
	}
      }
    }
  } 
  
  /**
   * This method highlights programmatically 'All games' node.
   */
  public void highlightAllGamesNode() {
    // Show list of all games
    if (config.getBoolean("Load_list_of_games")) {
      //****** Programatically highlight first tree node
      // Select path
      Object[] path = new Object[2];
      path[0] = tree.getModel().getRoot();
      path[1] = tree.getModel().getChild(path[0], 0);
      tree.setSelectionPath(new TreePath(path));
    }
  }
  
  /**
   * This method updates sound plugins.
   *
   * @param xmamePath The path to the xmame executable
   *
   * @see   com.miami.romeo.xmame.Xmame#setDigitalSoundPlugins(String)
   * @see   com.miami.romeo.xmame.Xmame#setSoundMixerPlugins(String)
   */
  public void setXmameSoundPlugins(String xmamePath) {
    xmame = new Xmame(romeo);

    if (romeo.options.dspBox != null &&
        romeo.options.mixer != null) {
      xmame.setDigitalSoundPlugins(xmamePath);
      xmame.setSoundMixerPlugins(xmamePath);
    }
  }

  /**
   * This method returns if a rom is runnable.
   *
   * @param gameName The game name
   * @param cloneOf  The original version of the given game
   * @param driver   The driver used by the game
   *
   * @return         <code>True</code> if the game is runnable;
   *                 <code>false</code> otherwise.
   *
   * @see            #getRomDirs()
   * @see            #gameExists(String[], String)
   * @see            com.miami.romeo.xmame.Xmame#romIsCorrect(String, String, String)
   * @see            #romNeedsKicker(String)
   * @see            #kickerIsAvailable(String[], String, String)
   * @see            #originalGameExists(String[], String)
   * @see            #appendMessage(String, String)
   */
  public boolean gameIsRunnable(String gameName, String cloneOf,
                                String driver) {
    boolean playable = true;

    String[] romDir = getRomDirs();

    // First of all, check if the game exists
    if (gameExists(romDir, gameName)) {
      String romPath = getGamePath(romDir, gameName);

      xmame = new Xmame(romeo);

      if (xmame.romIsCorrect(getXmamePath(), romPath, gameName)) {
        // Verify if the game is original
	if (cloneOf.equals("-")) {
	  // Verify if rom needs extra files
          if (romNeedsKicker(driver)) {
            if (kickerIsAvailable(romDir, gameName, driver))
	      playable = true;
	    else {
	      appendMessage("Error", gameName + " needs " + driver + ".zip!");
	      playable = false;
	    }
	  }
	  else
	    playable = true;
	}
	else {
          if (originalGameExists(romDir, cloneOf)) {
            // Verify if extra files are needed
            if (romNeedsKicker(driver)) {
              if (kickerIsAvailable(romDir, gameName, driver))
	        playable = true;
	      else {
	        appendMessage("Error", gameName + " needs " +
		              driver + ".zip!");
	        playable = false;
	      }
	    }
	    else
	      playable = true;
	  }
          else {
            appendMessage("Error", gameName + " needs " +
	                  cloneOf + ".zip");
	    playable = false;
	  }
	}
      }
      else {
        appendMessage("Error", gameName + " is incorrect");
	playable = false;
      }
    }
    else {
      appendMessage("Error", gameName + " not available.");
      playable = false;
    }

    // return if the game is runnable
    return playable;
  }

  /**
   * This method returns the rom dir paths.
   *
   * @return The rom dir paths
   *
   * @see    com.miami.romeo.config.Utils#parseValues(String, char)
   *
   * @since  0.1.3pre2
   */
  public String[] getRomDirs() {
    config = new Config(new File(System.getProperty("user.home") +
                                 File.separator + ".romeo4xmame" +
				 File.separator + "Romeo.ini"));

    return utils.parseValues(config.get("xmame.rom_dir"), ';');
  }

  /**
   * This method returns true if the given game exists, false otherwise.
   *
   * @param romDir   The rom dirs
   * @param gameName The game name
   *
   * @return         <code>True</code> if the given game exists,
   *                 <code>false</code> otherwise
   *
   * @since          0.1.3pre2
   */
  public boolean gameExists(String[] romDir, String gameName) {
    boolean existing = false;
    File f = null;

    for (int i = 0; i < romDir.length; i++) {
      f = new File(romDir[i] + gameName + ".zip");

      if (f.exists()) {
        existing = true;
        break;
      }
    }

    return existing;
  }

  /**
   * This method returns the rom dir path where the given game has been
   * stored in.
   *
   * @param romDir   The rom dirs
   * @param gameName The game name
   *
   * @return         The path to the given game
   *
   * @since          0.1.3pre2
   */
  public String getGamePath(String[] romDir, String gameName) {
    String gamePath = new String();
    File f = null;

    for (int i = 0; i < romDir.length; i++) {
      f = new File(romDir[i] + gameName + ".zip");

      if (f.exists()) {
        gamePath = romDir[i];
        break;
      }
    }

    return gamePath;
  }

  /**
   * This method verifies if the given game needs a kicker.
   *
   * @param driver The driver used by a given game
   *
   * @return       <code>True</code> if the game needs a kicker;
   *               <code>false</code> otherwise.
   *
   * @since        0.1.3pre1
   */
  public boolean romNeedsKicker(String driver) {
    if (driver.equals("neogeo"))
      return true;
    else if (driver.equals("decocass"))
      return true;
    else if (driver.equals("playch10"))
      return true;
    else if (driver.equals("cvs"))
      return true;
    else if (driver.equals("pgm"))
      return true;
    else if (driver.equals("skns"))
      return true;
    else if (driver.equals("konamigx"))
      return true;
    else
      return false;
  }

  /**
   * This method verifies if the kicker needed by the given game
   * is available.
   *
   * @param romDir    The path to the rom dirs
   * @param gameName  The game name
   * @param driver    The driver used by the game
   *
   * @return          <code>True</code> if the kicker is available;
   *                  <code>false</code> otherwise.
   *
   * @see             #kickerOK(String[], String)
   */
  public boolean kickerIsAvailable(String[] romDir,
                                   String gameName,
                                   String driver) {
    if (driver.equals("neogeo")) 
      return kickerOK(romDir, "neogeo");
    else if (driver.equals("decocass"))
      return kickerOK(romDir, "decocass");
    else if (driver.equals("playch10"))
      return kickerOK(romDir, "playch10");
    else if (driver.equals("cvs"))
      return kickerOK(romDir, "cvs");
    else if (driver.equals("pgm"))
      return kickerOK(romDir, "pgm");
    else if (driver.equals("skns"))
      return kickerOK(romDir, "skns");
    else if (driver.equals("konamigx"))
      return kickerOK(romDir, "konamigx");
    else
      return false;
  }

  /**
   * This method returns true if the kicker exists, false otherwise.
   *
   * @param romDir     The rom dirs
   * @param kickerName The kicker name
   *
   * @return           <code>True</code> if the kicker exists, 
   *                   <code>false</code> otherwise
   *
   * @since            0.1.3pre2
   */
  public boolean kickerOK(String[] romDir, String kickerName) {
    boolean ok = false;

    for (int i = 0; i < romDir.length; i++) {
      File kicker = new File(romDir[i] + kickerName + ".zip");

      if (kicker.exists()) {
        ok = true;
        break;
      }
    }

    return ok;
  }

  /**
   * This method returns true if the given game exists, false otherwise.
   *
   * @param romDir   The rom dirs
   * @param gameName The game name
   *
   * @return         <code>True</code> if the given game exists,
   *                 <code>false</code> otherwise
   *
   * @since          0.1.3pre2
   */
  public boolean originalGameExists(String[] romDir, String gameName) {
    boolean existing = false;
    File f = null;

    for (int i = 0; i < romDir.length; i++) {
      // The game is a clone: verify if the original rom is available
      f = new File(romDir[i] + cloneOf + ".zip");

      if (f.exists()) {
        existing = true;
        break;
      }
    }

    return existing;
  }
  
  /**
   * This method returns the name of the list of games.
   *
   * @param xmamePath The path to the xmame executable
   *
   * @return          <code>String</code>
   *
   * @see             com.miami.romeo.xmame.Xmame#getVersion(String)
   */
  protected String getNameOfList(String xmamePath) {
    // Path
    String listName = "";

    // The user has set a valid xmame executable
    if (!xmamePath.equals("<NOT SET>")) {
      // Call the class to dialog with xmame
      xmame = new Xmame(romeo);
      // Get xmame version
      String xmameVersion = xmame.getVersion(xmamePath);

      // Parse the returned version string to know which
      // list we need
      int idx_p1  = xmameVersion.indexOf('(');
      int idx_p2  = xmameVersion.indexOf(')');
      int VERSION = xmameVersion.indexOf("version");
      int idx_p3  = xmameVersion.lastIndexOf('(');
      int idx_p4  = xmameVersion.lastIndexOf(')');

      listName = "list-" + xmameVersion.substring(idx_p1 + 1, idx_p2) +
                 "-" + xmameVersion.substring(VERSION + 7, idx_p3).trim() + 
                 "-" + xmameVersion.substring(idx_p3, idx_p4 + 1);
    }

    // Return the path
    return listName;
  }

  /**
   * This method returns the xmame executable file path.
   *
   * @return <code>String</code>
   */
  protected String getXmamePath() {
    // Get the path to the xmame executable selected by the user
    return xmameComboBox.getSelectedItem().toString();
  }

  /**
   * This method appends a line to the message area in this way:
   * date + message.
   *
   * @param type    The type of message to display
   * @param message The message to display
   *
   * @see           #scrollMessageAreaToBottom()
   */
  public void appendMessage(String type, String message) {
    // Seconds, minutes, hours...
    int s = 0, m = 10, h = 10;
    String today;
    MutableAttributeSet textStyle = null;
    MutableAttributeSet iconStyle = null;
    String line = null;

    // Date
    currentDate = new Date();
    SimpleDateFormat formatter = new SimpleDateFormat("s", Locale.getDefault());
    try {
      s = Integer.parseInt(formatter.format(currentDate));
    }
    catch (NumberFormatException n) {
      s = 0;
    }
    formatter.applyPattern("m");
    try {
      m = Integer.parseInt(formatter.format(currentDate));
    }
    catch (NumberFormatException n) {
      m = 10;
    }
    formatter.applyPattern("h");
    try {
      h = Integer.parseInt(formatter.format(currentDate));
    }
    catch (NumberFormatException n) {
      h = 10;
    }
    // today contains day, month, year...
    formatter.applyPattern("EEE MMM dd HH:mm:ss yyyy");
    today = formatter.format(currentDate);

    StyledDocument doc = messageArea.getStyledDocument();

    if (type.equals("Error")) {
      iconStyle = new SimpleAttributeSet();
      StyleConstants.setIcon(iconStyle, utils.createImageIcon16x16("Remove.png"));
      //  Define a keyword attribute for error messages
      textStyle = new SimpleAttributeSet();
      StyleConstants.setForeground(textStyle, Color.red);
      StyleConstants.setItalic(textStyle, false);
    }
    else if (type.equals("Info")) {
      iconStyle = new SimpleAttributeSet();
      StyleConstants.setIcon(iconStyle, utils.createImageIcon16x16("Info.png"));
      //  Define a keyword attribute for information messages
      textStyle = new SimpleAttributeSet();
      StyleConstants.setForeground(textStyle, Color.blue);
      StyleConstants.setItalic(textStyle, false);
    }
    else {
      iconStyle = new SimpleAttributeSet();
      StyleConstants.setIcon(iconStyle, utils.createImageIcon16x16("List.png"));
      //  Define a default attribute
      textStyle = new SimpleAttributeSet();
      StyleConstants.setForeground(textStyle, new Color(204, 0, 204));
      StyleConstants.setFontSize(textStyle, 14);
      StyleConstants.setItalic(textStyle, false);

      //  Highlight some keywords
      //doc.setCharacterAttributes(36, 42, style, false);
      //doc.setCharacterAttributes(46, 57, style, false);
    }

    if (message.startsWith("This message area"))
      line = " " + message + "\n";
    else
      line = " [" + today + "]: " + message + "\n";

    //Load the text pane with styled text.
    try {
      doc.insertString(doc.getLength(), " ", iconStyle);
      doc.insertString(doc.getLength(), line, textStyle);
    } catch (BadLocationException ble) {
      System.err.println("Couldn't insert initial text.");
    }
    
    if (scrollMessages != null)
      scrollMessageAreaToBottom();

    lastDate = today;
    currentDate = null;
  }
  
  /**
   * This method scrolls the message area to the bottom.
   */
  public void scrollMessageAreaToBottom() {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        scrollMessages.getVerticalScrollBar().setValue(
                           scrollMessages.getVerticalScrollBar().getMaximum());
      }
    });
  }

  /**
   * This method sets the values for the progress bars contained in
   * the table.
   *
   * @param gameName The game name
   * @param MAX      The maximum value
   */
  public void setNumberOfPlayedMatches(String gameName, int MAX) {
    String xmamePath = getXmamePath();
    String listName = getNameOfList(xmamePath);

    // Call this class to save favorite status in a specific text file
    config = new Config(new File(System.getProperty("user.home") +
                                 File.separator + ".romeo4xmame" +
	                         File.separator + "played" +
				 listName.substring(4) + ".ini"));

    int n = config.getInt(gameName);
    int n2;

    if (n > 0) {
      n2 = n + 1;
      config.add(gameName, n2);
    }
    else {
      n2 = 1;
      config.add(gameName, n2);
    }

    config.save();

    // Use data loaded into a memory stack
    Vector data = row;

    for (int i = 0; i < data.size(); i++) {
      Object[] line = (Object[])data.elementAt(i);
      if (line[2].equals(gameName))
        line[11] = n2 + "";

	row.setElementAt(line, i);
    }

    dataio = new DataIO(romeo);
    dataio.updateListOfGames(row, xmamePath, listName);

    // Renderer for tables cells with a progress bar
    iRenderer = createIndicatorCellRenderer();

    String header = utils.getString("Table.column_played");

    for (int i = 0; i < table.getColumnCount(); i++) {
      if (table.getColumnName(i).equals(header)) {
	columnModel1.getColumn(i).setCellRenderer(iRenderer);
        break;
      }
    }

    for (int i = 0; i < table.getRowCount(); i++) {
      if ((table2.getValueAt(i, 0).toString().equals(gameName))) {
        table.setValueAt(checkMinMax(new Integer(n2)), i, 10);
        break;
      }
    }
    table.repaint();
  }

  /**
   * This method checks min/max values for the progress bars in the 
   * main table.
   *
   * @param value The value to check
   *
   * @return Integer
   */
  public Integer checkMinMax(Integer value) {
    int MIN = 0;
    config = new Config(new File(System.getProperty("user.home") +
                                 File.separator + ".romeo4xmame" +
				 File.separator + "Romeo.ini"));

    String xmamePath = getXmamePath();
    int  MAX = config.getInt(getNameOfList(xmamePath) + ".total_played");
    int intValue = value.intValue();
    if (intValue < MIN) {
      intValue = MIN;
    } else if (MAX < intValue) {
      intValue = MAX;
    }
    return new Integer(intValue);
  }
  
  /**
   * This method parses an internet url set by the user to
   * download a file (rom, snap etc.), returning a string
   * with all chars '$' substituted by the file name.
   *
   * @param url      The url to parse
   * @param fileName The file name
   *
   * @return         <code>String</code>
   */
  public String parseURL(String url, String fileName) {
    // The end of the url containing chars '$'
    String partialUrl;
    // Parsed string
    String parsedUrl = new String();
    // The string returned at the end of this process
    String finalUrl = new String();
    // First occurrence of the char '$'
    int idx_$ = url.indexOf('$');
    //System.out.println("Char . alla posizione: " + idx_p);

    // Character '$' has been found
    if (idx_$ > 0) {
      parsedUrl = url.substring(0, idx_$);
      // Get last occurrence of char '$'
      int last_idx_$ = url.lastIndexOf('$');

      // Recursively substitute all chars '$' with the file name
      while (idx_$ <= last_idx_$) {
	//System.out.println("Char $ alla posizione: " + idx_$);
	partialUrl = url.substring(idx_$ + 1);
	parsedUrl = parsedUrl + fileName + partialUrl;
	int new_idx_$ = partialUrl.indexOf('$');
	if (new_idx_$ > 0) {
	  idx_$ = idx_$ + new_idx_$ + 1;
	  parsedUrl = parsedUrl.substring(0, parsedUrl.indexOf('$'));
	}
	// no more occurrences: break!
	else
	  break;
      }

      finalUrl = parsedUrl;
    }
    else {
      finalUrl = url;
    }

    // return the correct url
    return finalUrl;
  }
  
  // *******************************************************
  // **********************  Misc  *************************
  // *******************************************************

  /**
   * This method creates a dialog box to show info about a game.
   *
   * @param title    The box' s caption
   * @param gameName The game name
   * @param cloneOf  The original version of a given game
   * @param lines    The array of data to show
   */
  public void createInfoDialog(String title, String gameName, String cloneOf,
                               String[] lines) {
    // JPanel panel = new JPanel(new BorderLayout());
    JPanel panel = new InfoPanel(gameName, cloneOf, lines);

    infoDialog = new JDialog(romeo.getFrame(), title, true);
    infoDialog.getContentPane().add(panel, BorderLayout.CENTER);
    
    JPanel buttonPanel = new JPanel();
    buttonPanel.setOpaque(false);
    JButton button = (JButton) buttonPanel.add(
      new JButton(utils.getString("AboutBox.ok_button_text"))
    );
    panel.add(buttonPanel, BorderLayout.SOUTH);
    button.addActionListener(new OkAction(infoDialog));


    infoDialog.setSize(450, 250);
    infoDialog.setResizable(false);
    Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
    Rectangle frameDim = infoDialog.getBounds();
    infoDialog.setLocation((screenDim.width  - frameDim.width)  / 2,
                           (screenDim.height - frameDim.height) / 2);
			   
    //  Handle escape key to close the dialog
    KeyStroke escapeKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
    Action escapeAction = new AbstractAction(){
      public void actionPerformed(ActionEvent e) {
        infoDialog.setVisible(false);
      }
    };
    infoDialog.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
                                                     escapeKeyStroke, "ESCAPE");
    infoDialog.getRootPane().getActionMap().put("ESCAPE", escapeAction);

    infoDialog.show();
  }
  
  /**
   * This class creates a JPanel containing a JList to list game info.
   */
  class InfoPanel extends JPanel {
    String gameName;
    String[] lines;

    JList infoList = null;

    /**
     * Constructor
     *
     * @param gameName The game name
     * @param cloneOf  The original version of a given game
     * @param lines    The array of data to show
     */
    public InfoPanel(final String gameName,
                     final String cloneOf,
                     String[] lines) {
      this.gameName = gameName;
      this.lines = lines;

      setLayout(new BorderLayout());
      setOpaque(false);

      // Use a list model that allows for updates
      DefaultListModel infoListModel = new DefaultListModel();

      for (int i = 0; i < lines.length; i++)
        infoListModel.addElement(lines[i]);

      infoList = new JList(infoListModel);
      infoList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      InfoListCellRenderer renderer = new InfoListCellRenderer();
      if (lines[0].startsWith(utils.getString("Table.column_name") + ":"))
        renderer.setTabs(new int[] {200, 200});
      else
        renderer.setTabs(new int[] {200, 200, 200, 200});
      infoList.setCellRenderer(renderer);

      infoList.addMouseListener(new MouseAdapter() {
        public void mouseClicked(MouseEvent e) {
	  String line = infoList.getSelectedValue().toString();
	  int last_idx_gameName = line.lastIndexOf(gameName);

          if (last_idx_gameName > 0) {
	    String value = line.substring(0, last_idx_gameName).trim();
	    for (int i = 0; i < table2.getRowCount(); i++) {
              if (table2.getValueAt(i, 0).toString().equalsIgnoreCase(value)) {
	        highlightRow(table2, i, 0);
	        break;
              }
            }
	  }
	  else {
	    int last_idx_cloneOf = line.lastIndexOf(cloneOf);
	    
	    if (last_idx_cloneOf > 0) {
	      String value = line.substring(last_idx_cloneOf).trim();
	      for (int i = 0; i < table2.getRowCount(); i++) {
                if (table2.getValueAt(i, 0).toString().equalsIgnoreCase(value)) {
	          highlightRow(table2, i, 0);
	          break;
              }
            }
	  }
	  }
        }
      });

      JScrollPane sp = new JScrollPane(infoList);
      add(sp, BorderLayout.CENTER);
    }
  }
  
  /**
   * This method highlights a row in the list of games.
   *
   * @param table  The table where to highlight the row in
   * @param row    The row to highlight
   * @param column The column where to scroll the table
   */
  public void highlightRow(JTable table, int row, int column) {
    Rectangle cell = table.getCellRect(row, column, true);
    Point p = new Point(cell.x, cell.y);

    // Programatically scroll tables horizontally
    headScroll.getModel().setValue(p.x);
    // Programatically scroll tables vertically
    scrollPanel.scrollRectToVisible(cell);
    // programatically highlight the row
    table.setRowSelectionInterval(row, row);
  }

  /**
   * This class closes the box to show info about a game.
   */
  class OkAction extends AbstractAction {
    JDialog infoDialog;

    protected OkAction(JDialog infoDialog) {
      super("OkAction");
      this.infoDialog = infoDialog;
    }

    public void actionPerformed(ActionEvent e) {
      infoDialog.setVisible(false);
    }
  }
}  // That' s all, folks.
