
/*==============================================================================
 *
 * (#)Info.java
 * created      2003/03/25
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *==============================================================================
 */

package com.miami.romeo.main;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.beans.*;
import java.net.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;

import javax.accessibility.*;

import com.miami.romeo.config.*;
import com.miami.romeo.graphics.DialogSeparator;
import com.miami.romeo.xmame.*;

/**
 * This class extends JPanel to create a container for a JDesktopPane.
 *
 * @author       $Author: Stefano Bolli
 * @version      $Revision: 1.2 $Date: 2003/10/29 00:11:35 $
 */
public class Info extends JPanel {
  Romeo romeo        = null;
  // Utilities
  Utils utils        = new Utils();
  // Configuration file
  Config config      = null;
  // Dialog separator
  DialogSeparator ds = null;
  // A class to dialog with xmame
  Xmame xmame        = null;

  // Used for cascading
  int windowCount = 0;
  // Our desktop Pane
  JDesktopPane desktop = null;
  // Custom DesktopManager implementation that updates WindowWatcher
  // whenever any of its methods are called
  WindowManager wm     = null;
  // The viewport of the scrollpane that will contain our desktop
  JViewport viewport   = null;

  // Palette
  public JInternalFrame palette = null;

  // Palette options
  public Integer PALETTE_LAYER = new Integer(4);

  // Palette location
  public int PALETTE_X;
  public int PALETTE_Y;
  // Keep track of the most recent x coordinate of the desktop 
  // scrollpane 's view position
  public int wmX;
  // Keep track of the most recent x coordinate of the desktop 
  // scrollpane' s view position
  public int wmY;

  // Palette size
  public int PALETTE_WIDTH  = 235;
  public int PALETTE_HEIGHT = 275;

  //******* Palette' s components
  // Button to show images
  JButton b1 = null;
  // Button to show info retrieved from the internet
  JButton b2 = null;
  // Button to show the history of a selected game
  JButton b3 = null;
  // Button to show detailed info about a game
  JButton b4 = null;

  // Check boxes: if selected Romeo will show:
  // snap
  JCheckBox snapBox    = null;
  // flyer
  JCheckBox flyerBox   = null;
  // marquee
  JCheckBox marqueeBox = null;
  // cabinet photo
  JCheckBox cabinetBox = null;

  // Text field to read info from history.dat
  public JTextField historyField = null;

  // Component to read info retrieved from the internet
  JEditorPane finalpane = null;

  // File chooser...
  JFileChooser fileChooser = new JFileChooser();

  /**
   * Constructor
   *
   * @param romeo The caller
   */
  public Info(Romeo romeo) {
    this.romeo = romeo;

    setLayout(new BorderLayout());
    
    // We create a new JDesktopPane and place it inside a JScrollPane. Then
    // we create a new WindowManager (see below) and pass it a reference to
    // our desktop. We then tell our desktop that this WindowManager is a
    // DesktopManager implementation and it should be used to manage our
    // internal frames. This is done with JDesktopPane?s setDesktopManager()
    // method. We then place our WindowManager?s WindowWatcher in our
    // desktop' s PALETTE_LAYER. This will guarantee that it is always
    // displayed over all internal frames
    desktop = new JDesktopPane();
    JScrollPane scroller = new JScrollPane();
    wm = new WindowManager(desktop);
    desktop.add(wm.getWindowWatcher(), JLayeredPane.PALETTE_LAYER);
    wm.getWindowWatcher().setBounds(555, 5, 200, 150);
    
    // A custom JViewport is constructed with an overriden setViewPosition() 
    // method. This method is responsible for keeping our WindowWatcher in 
    // the same place as we scroll the desktop. Each time the view is changed 
    // we reposition the WindowWatcher to give the impression that it lies 
    // completely above our desktop and is unaffected by scrolling. Basically, 
    // this code just computes the difference between the current and most 
    // recent viewport position, and adds this difference to the coordinates 
    // of the WindowWatcher. We then use this JViewport as the viewport for 
    // the JScrollPane our desktop is contained in using JScrollPane?s setView() 
    // method.
    viewport = new JViewport() {
      public void setViewPosition(Point p) { 
        super.setViewPosition(p); 
        wm.getWindowWatcher().setLocation(wm.getWindowWatcher().getX() +
                                          (getViewPosition().x - wmX),
                                           wm.getWindowWatcher().getY() +
                                          (getViewPosition().y - wmY));
        wmX = getViewPosition().x;
        wmY = getViewPosition().y;
      }
    };
    viewport.setView(desktop);
    scroller.setViewport(viewport);
    
    // Next we construct a ComponentAdapter and attach it to our viewport. 
    // We override its componentResized() method to move WindowWatcher to 
    // the topwhenever the viewport is resized. This is done so that WindowWatcher 
    // will never disappear from our view when the application is resized.
    ComponentAdapter ca = new ComponentAdapter() {
      JViewport view = viewport;
      public void componentResized(ComponentEvent e) {
        wm.getWindowWatcher().setLocation(view.getViewPosition().x +
	                                  view.getWidth() - 
					  wm.getWindowWatcher().getWidth() - 15,
                                          view.getViewPosition().y + 5);
      }
    };
    viewport.addComponentListener(ca);

    add(scroller, BorderLayout.CENTER);
    
    desktop.setPreferredSize(new Dimension(1600, 1200));

    // Create the "frame maker" palette
    createInternalFramePalette();
  }

  /**
   * This method creates an internal frame palette.
   *
   * @return <code>JInternalFrame</code>
   */
  public JInternalFrame createInternalFramePalette() {
    config = new Config(new File(System.getProperty("user.home") + 
                                 File.separator + ".romeo4xmame" + 
				 File.separator + "Romeo.ini"));

    palette = new JInternalFrame(
	      utils.getString("InternalFrameDemo.palette_label"));
    palette.putClientProperty("JInternalFrame.isPalette", Boolean.TRUE);
    palette.getContentPane().setLayout(null);
    PALETTE_X = config.getInt("Palette.LOCATION_X");
    PALETTE_Y = config.getInt("Palette.LOCATION_Y");
    palette.setBounds(PALETTE_X, PALETTE_Y, PALETTE_WIDTH, PALETTE_HEIGHT);
    palette.setResizable(false);
    palette.setIconifiable(true);
    palette.setFrameIcon(utils.createImageIcon16x16("Desktop.png"));
    desktop.add(palette, PALETTE_LAYER);

    // ******************************
    // * Create frame maker buttons *
    // ******************************
    b1 = new JButton(utils.createImageIcon32x32("Image.png"));
    b1.setBounds(10, 10, 45, 45);
    b1.setToolTipText(utils.getString(
                                  "InternalFrameDemo.palette_images_tooltip"));
    b1.addActionListener(new ShowImagesAction(this));

    b2 = new JButton(utils.createImageIcon32x32("Web.png"));
    b2.setBounds(65, 10, 45, 45);
    b2.setToolTipText(utils.getString(
                                  "InternalFrameDemo.palette_mamedk_tooltip"));
    b2.addActionListener(new ShowWebInfoAction(this));

    b3 = new JButton(utils.createImageIcon32x32("History.png"));
    b3.setBounds(120, 10, 45, 45);
    b3.setToolTipText(utils.getString(
                                 "InternalFrameDemo.palette_history_tooltip"));
    b3.addActionListener(new ShowHistoryAction(this));

    b4 = new JButton(utils.createImageIcon32x32("Filter.png"));
    b4.setBounds(175, 10, 45, 45);
    b4.setToolTipText(utils.getString(
                             "InternalFrameDemo.palette_detailedinfo_tooltip"));
    b4.addActionListener(new ShowDetailedInfoAction(this));

    // add frame maker buttons to panel
    JPanel p = new JPanel();
    p.setLayout(null);
    p.setBounds(0, 0, 230, 75);
    p.add(b1);
    p.add(b2);
    p.add(b3);
    p.add(b4);

    palette.getContentPane().add(p);

    // ************************************
    // * Create frame property checkboxes *
    // ************************************
    snapBox = new JCheckBox();
    snapBox.setBounds(10, 20, 90, 24);
    snapBox.setText("Snapshot");
    snapBox.setSelected(true);

    flyerBox = new JCheckBox();
    flyerBox.setBounds(110, 20, 90, 24);
    flyerBox.setText("Flyer");
    flyerBox.setSelected(true);

    marqueeBox = new JCheckBox();
    marqueeBox.setBounds(10, 54, 90, 24);
    marqueeBox.setText("Marquee");
    marqueeBox.setSelected(true);

    cabinetBox = new JCheckBox();
    cabinetBox.setBounds(110, 54, 90, 24);
    cabinetBox.setText("Cabinet");
    cabinetBox.setSelected(true);

    // add frame property checkboxes to panel
    JPanel p1 = new JPanel();
    p1.setLayout(null);
    p1.setBorder(BorderFactory.createTitledBorder(utils.getString(
				   "InternalFrameDemo.palette_images_select")));
    p1.setBounds(10, 75, 210, 90);
    p1.add(snapBox);
    p1.add(flyerBox);
    p1.add(marqueeBox);
    p1.add(cabinetBox);

    palette.getContentPane().add(p1);

    historyField = new JTextField(config.get("xmame.history.dat_path"));
    historyField.setBounds(10, 20, 161, 24);

    JPanel p2 = new JPanel();
    p2.setLayout(null);
    p2.setBorder(BorderFactory.createTitledBorder(
                 utils.getString("InternalFrameDemo.palette_history.dat_path")));

    JButton historyButton = new JButton();
    historyButton.setBounds(176, 20, 24, 24);
    historyButton.setIcon(utils.createImageIcon16x16("Fileopen.png"));
    historyButton.addActionListener(new openHistoryFile());

    p2.setBounds(10, 175, 210, 60);
    p2.add(historyField);
    p2.add(historyButton);

    palette.getContentPane().add(p2);

    palette.show();

    return palette;
  }
  
  // ********************
  // * Show game images *
  // ********************

  /**
   * This class shows images for a given game.
   */
  class ShowImagesAction extends AbstractAction {
    Info info;

    public ShowImagesAction(Info info) {
      this.info = info;
    }

    public void actionPerformed(ActionEvent e) {
      if (romeo.games.gameName != null) {
        config = new Config(new File(System.getProperty("user.home") + 
	                             File.separator + ".romeo4xmame" + 
				     File.separator + "Romeo.ini"));

        String[] prefixes = {
	                     config.get("xmame.snap_dir"),
                             config.get("xmame.flyer_dir"),
                             config.get("xmame.marquee_dir"),
                             config.get("xmame.cabinet_dir")
			    };

        String[] extensions = {
	                       config.get("xmame.snap_server"),
			       config.get("xmame.flyer_server"),
			       config.get("xmame.marquee_server"),
			       config.get("xmame.cabinet_server")
	                      };

        if ((!prefixes[0].equals("")) && (!prefixes[1].equals("")) &&
	    (!prefixes[2].equals("")) && (!prefixes[3].equals("")) &&
	    (!extensions[0].equals("")) && (!extensions[1].equals("")) &&
	    (!extensions[2].equals("")) && (!extensions[0].equals(""))) {
          String[] titles = {
	                     "Snapshot: ",
                             "Flyer: ",
                             "Marquee: ",
                             "Cabinet: "
	 	  	    };

          String imagePath;
          File image;

          String cloneOf = romeo.games.cloneOf;
          String gameName = romeo.games.gameName;
	  String game;

          boolean[] boxesSelected = {
                                     snapBox.isSelected(),
                                     flyerBox.isSelected(),
                                     marqueeBox.isSelected(),
	     			     cabinetBox.isSelected()
	                            };

          int[] idx_$ = {
	                 extensions[0].indexOf('$'),
		         extensions[1].indexOf('$'),
		         extensions[2].indexOf('$'),
		         extensions[3].indexOf('$'),
	                };

          for (int i = 0; i < prefixes.length; i++) {
	    String extension;

	    if (idx_$[i] > 0)
	      extension = extensions[i].substring(idx_$[i] + 1);
	    else
	      extension = ".png";

            imagePath = prefixes[i] + gameName + extension;
            image = new File(imagePath);
            game = gameName;

            if ((!image.exists()) && (!cloneOf.equals("-"))) {
              imagePath = prefixes[i] + cloneOf + extension;
              image = new File(imagePath);
              game = cloneOf;
            }


            int layer = 1;

            if (!prefixes[i].equals("")) {
              if (image.exists())  {
                if (boxesSelected[i]) {
                  ImageIcon img = new ImageIcon(imagePath);
                  info.createInternalImageFrame(img, new Integer(layer),
                                                img.getIconWidth(),
					        img.getIconHeight(),
                                                titles[i], game);
                }
              }
            }
          }
	}
	else {
	  JOptionPane.showMessageDialog(null,
                              utils.getString(
			      "Messagebox.software_not_configured"),
                              utils.getString("Messagebox.error_title"),
                              JOptionPane.ERROR_MESSAGE);
	}
      }
      else {
        JOptionPane.showMessageDialog(null,
                              utils.getString(
			      "Messagebox.error_nogameselected"),
                              utils.getString("Messagebox.error_title"),
                              JOptionPane.ERROR_MESSAGE);
      }
    }
  }
  
  /**
   * This method creates a generic internal frame and adds a scrollable 
   * image to it.
   *
   * @param icon     The image to show
   * @param layer    An integer specifying this internal frame's desktop layer
   * @param width    An integer giving the component's new width in pixels
   * @param height   An integer giving the component's new height in pixels
   * @param title    The component' s caption
   * @param gameName The name of the game selected by the user
   * @return         <code>JInternalFrame</code>
   */
  public JInternalFrame createInternalImageFrame(Icon icon, Integer layer,
                                                 int width, int height,
                                                 String title, String gameName) {
    JInternalFrame jif = new JInternalFrame(title + " " + gameName);

    // set properties
    jif.setClosable(true);
    jif.setMaximizable(true);
    jif.setIconifiable(true);
    jif.setResizable(true);
    jif.setFrameIcon(utils.createImageIcon16x16("Image.png"));

    jif.setBounds(20 * (windowCount%10), 20 * (windowCount%10), width, height);
    jif.setContentPane(new ImageScroller(this, icon, 0));

    windowCount++;

    desktop.add(jif, layer);

    // Set this internal frame to be selected
    try {
      jif.setSelected(true);
    } catch (java.beans.PropertyVetoException e2) {
    }

    jif.show();

    return jif;
  }

  /**
   * This class extends JScrollPane 
   */
  class ImageScroller extends JScrollPane {
    public ImageScroller(Info info, Icon icon, int layer) {
      super();

      JPanel p = new JPanel();
      p.setBackground(Color.white);
      p.setLayout(new BorderLayout() );

      p.add(new JLabel(icon), BorderLayout.CENTER);

      getViewport().add(p);
    }

    public Dimension getMinimumSize() {
      return new Dimension(25, 25);
    }
  }
  
  // *****************
  // * Show web info *
  // *****************

  /**
   * This class retrieves game info from the internet.
   */
  class ShowWebInfoAction extends AbstractAction {
    Info info;

    public ShowWebInfoAction(Info info) {
      this.info = info;
    }

    public void actionPerformed(ActionEvent e) {
      if (romeo.games.gameName != null) {
        if (config.get("User.browser").length() > 0) {
          Thread thread = new Thread() {
            public void run() {
              utils.goToUrl("http://caesar.logiqx.com/html/emus/multi/mame/" +
                            romeo.games.gameName + ".shtml");
	    }
          };
          thread.start();
        }
        else
          info.createInternalWebFrame(new Integer(2), 400, 300);
      }
      else {
        JOptionPane.showMessageDialog(null,
                             utils.getString("Messagebox.error_nogameselected"),
                             utils.getString("Messagebox.error_title"),
                             JOptionPane.ERROR_MESSAGE);
      }
    }
  }
  
  /**
   * Create an internal frame to read info about a game
   * retrieved over the Internet.
   */
  public JInternalFrame createInternalWebFrame(Integer layer, int width,
                                               int height) {
    JInternalFrame jif = new JInternalFrame("Mame Explorer - " +
                                            romeo.games.gameName);

    // set properties
    jif.setClosable(true);
    jif.setMaximizable(true);
    jif.setIconifiable(true);
    jif.setResizable(true);
    jif.setFrameIcon(utils.createImageIcon16x16("Mamedk.png"));

    jif.setBounds(590, 270, width, height);
    jif.setContentPane(new InfoScroller(this, 6));

    desktop.add(jif, layer);

    // Set this internal frame to be selected
    try {
      jif.setSelected(true);
    } catch (java.beans.PropertyVetoException e2) {
    }

    jif.show();

    return jif;
  }

  class InfoScroller extends JScrollPane {
    public InfoScroller(Info info, int layer) {
      super();

      String prefix = "http://caesar.logiqx.com/html/emus/multi/mame/";
      String gameName   = romeo.games.gameName;
      String suffix = ".shtml";

      JEditorPane pane;

      JPanel p = new JPanel();
      p.setBackground(Color.white);
      p.setLayout(new BorderLayout());

      try {
        pane= new JEditorPane(prefix + gameName + suffix);
        pane.setEditable(false);
        finalpane = new JEditorPane();
        finalpane = pane;

        // Are there links? Let's follow them...
        pane.addHyperlinkListener(new HyperlinkListener() {
          public void hyperlinkUpdate(HyperlinkEvent r) {
            try {
              if (r.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
                finalpane.setPage(r.getURL());
            }
            catch (Exception e) {
              JOptionPane.showMessageDialog(Info.this,
	                                    "Error: "+ e.toString(),
	                                    "Warning", JOptionPane.WARNING_MESSAGE);
            }
          }
        });

        p.add(new JScrollPane(pane), BorderLayout.CENTER);
	getViewport().add(p);

      }
      catch(IOException ioe) {
        JOptionPane.showMessageDialog(Info.this,
	                              "Error: "+ ioe.toString(),
	                              "Warning", JOptionPane.WARNING_MESSAGE);
      }
    }

    public Dimension getMinimumSize() {
      return new Dimension(25, 25);
    }
  }
  
  // *********************
  // * Show history info *
  // *********************

  /**
   * Show history info for a given game taken from history.dat.
   */
  class ShowHistoryAction extends AbstractAction {
    Info info;

    public ShowHistoryAction(Info info) {
      this.info = info;
    }

    public void actionPerformed(ActionEvent e) {
      if (romeo.games.gameName != null) {
        String gameName = romeo.games.gameName;

	config = new Config(new File(System.getProperty("user.home") + 
	                             File.separator + ".romeo4xmame" + 
				     File.separator + "Romeo.ini"));

	String historydatPath = config.get("xmame.history.dat_path");

	if ((!historydatPath.equals("")) &&
	    (!historyField.getText().equals(""))) {
          info.createInternalHistoryFrame(new Integer(3), 400, 300,
	                                  gameName, historydatPath);
        }
	else {
          JOptionPane.showMessageDialog(null,
	                                utils.getString(
					"Messagebox.software_not_configured"),
                                        utils.getString(
					"Messagebox.error_title"),
                                        JOptionPane.ERROR_MESSAGE);
        }
      }
      else {
        JOptionPane.showMessageDialog(null,
                              utils.getString(
			      "Messagebox.error_nogameselected"),
                              utils.getString(
			      "Messagebox.error_title"),
                              JOptionPane.ERROR_MESSAGE);
      }
    }
  }
  
  /**
   * Create an internal frame to show history of a selected game
   */
   public JInternalFrame createInternalHistoryFrame(Integer layer, int width,
                                                    int height,
                                                    String gameName,
						    String historydatPath) {

     JInternalFrame jif = new JInternalFrame("History: " + gameName);

     // set properties
     jif.setClosable(true);
     jif.setMaximizable(true);
     jif.setIconifiable(true);
     jif.setResizable(true);
     jif.setFrameIcon(utils.createImageIcon16x16("Info.png"));

     jif.setBounds(20, 270, width, height);
     jif.setContentPane(new HistoryScroller(this, 0, gameName, historydatPath));

     desktop.add(jif, layer);

     // Set this internal frame to be selected
     try {
       jif.setSelected(true);
     } catch (java.beans.PropertyVetoException e2) {
     }

     jif.show();

     return jif;
  }

  class HistoryScroller extends JScrollPane {
    String gameName;
    String clone;
    String historydatPath;
    JTextArea history = null;

    String line;

    public HistoryScroller(Info info, int layer,
                           final String gameName,
			   final String historydatPath) {
      super();

      JPanel p = new JPanel();
      p.setBackground(Color.white);
      p.setLayout(new BorderLayout() );

      history = new JTextArea();
      history.setMargin(new Insets(5,5,5,5));
      history.setBorder(new SoftBevelBorder(BevelBorder.LOWERED));
      history.setEditable(false);

      Thread thread = new Thread() {
        public void run() {
          Runnable runner = new Runnable() {
            public void run() {
              try {
	        clone = romeo.games.cloneOf;

                LineNumberReader nr = new LineNumberReader(
                                      new FileReader (new File(historydatPath)));

                // Game is original
		if (clone.equals("-")) {
                  while ((line = nr.readLine()) != null) {
                    // System.out.println("reading in: ->" + line + "<-");
		    if (line.startsWith("$info=" + gameName)) {
                      int lineNumber = nr.getLineNumber();
                      nr.mark(lineNumber + 2);
                      while ((line = nr.readLine()) != null) {
                        if (line.startsWith("$"))
                          history.append("\n");
                        else
                          history.append(line + "\n");
                        if (line.equals("$end"))
                          break;
                      }
                    }
                  }
                }
                else {
                  int comma;
                  int lastComma;
                  int comma2;

                  while ((line = nr.readLine()) != null) {
                    if (line.startsWith("$info=" + clone)) {
                      // Parse line
                      String parsed = line;
                      lastComma = parsed.lastIndexOf(',');
                      comma = parsed.indexOf(',');
                      parsed = parsed.substring(comma + 1);

                      while (comma < lastComma) {
                        comma2 = parsed.indexOf(',');
                        String cloneof = parsed.substring(0, comma2);

                        if (cloneof.equals(gameName)) {
                          int lineNumber = nr.getLineNumber();
                          nr.mark(lineNumber + 2);
                          while ((line = nr.readLine()) != null) {
                            if (line.startsWith("$"))
                              history.append("\n");
                            else
                              history.append(line + "\n");
                            if (line.equals("$end"))
                              break;
                          }
                        }

                        parsed = parsed.substring(comma2 + 1);
                        comma = comma + comma2 + 1;
                      }
                    }
                  }
                }

                nr.close();

                if (history.getText().equals("")) {
                  history.append("\n" + "No information found!");
                }
              }
              catch(IOException ioe) {
                System.err.println(ioe.getMessage());
              }
	    }
          };
          SwingUtilities.invokeLater(runner);
        }
      };
      thread.start();

      p.add(new JScrollPane(history), BorderLayout.CENTER);

      getViewport().add(p);
    }

    public Dimension getMinimumSize() {
      return new Dimension(25, 25);
    }
  }
  
  // **********************
  // * Show detailed info *
  // **********************
  
  /**
   * Show detailed game info.
   */
  class ShowDetailedInfoAction extends AbstractAction {
    Info info;

    public ShowDetailedInfoAction(Info info) {
      this.info = info;
    }

    public void actionPerformed(ActionEvent e) {
      String gameName  = romeo.games.gameName;

      if (gameName != null) {
        String xmamePath = romeo.games.getXmamePath();
        xmame = new Xmame(romeo);
        String detailedInfo = xmame.getDetailedInfo(xmamePath, gameName);
	String colors = xmame.getRomColors(xmamePath, gameName);
	String[] clones = xmame.getClonesOfGame(xmamePath, gameName);
        info.createInternalDetailedInfoFrame(new Integer(3), 600, 150,
	                                     gameName, detailedInfo, colors
					     /*clones*/);
      }
      else {
        JOptionPane.showMessageDialog(null,
                             utils.getString("Messagebox.error_nogameselected"),
                             utils.getString("Messagebox.error_title"),
                             JOptionPane.ERROR_MESSAGE);
      }
    }
  }
  
  /**
   * Create an internal frame to read info about a game
   * retrieved from xmame.
   */
  public JInternalFrame createInternalDetailedInfoFrame(Integer layer, 
                                                        int width, int height,
							String gameName,
							String detailedInfo,
							String colors
							/*String[] clones*/) {
    JPanel panel = new JPanel(new BorderLayout());
    panel.setBorder(new EmptyBorder(5, 5, 5, 5));
    
    // Panel
    JPanel panel0 = new JPanel(new BorderLayout());

    // Panel containing cpu data
    JPanel panel1 = new JPanel(new BorderLayout());

    // Dialog separator
    ds = new DialogSeparator("Cpu");
    panel1.add(ds, BorderLayout.NORTH);

    JPanel cpuPanel = new JPanel(new GridLayout(2, 8));

    // cpu 1
    JLabel l = new JLabel(" Cpu 1: ");
    cpuPanel.add(l);

    JTextField tf = new JTextField(detailedInfo.substring(19, 28).trim());
    tf.setEditable(false);
    cpuPanel.add(tf);

    // cpu 2
    l = new JLabel(" Cpu 2: ");
    cpuPanel.add(l);

    tf = new JTextField(detailedInfo.substring(28, 37).trim());
    tf.setEditable(false);
    cpuPanel.add(tf);

    // cpu 3
    l = new JLabel(" Cpu 3: ");
    cpuPanel.add(l);

    tf = new JTextField(detailedInfo.substring(37, 46).trim());
    tf.setEditable(false);
    cpuPanel.add(tf);

    // cpu 4
    l = new JLabel(" Cpu 4: ");
    cpuPanel.add(l);

    tf = new JTextField(detailedInfo.substring(46, 55).trim());
    tf.setEditable(false);
    cpuPanel.add(tf);

    // cpu 5
    l = new JLabel(" Cpu 5: ");
    cpuPanel.add(l);

    tf = new JTextField(detailedInfo.substring(55, 64).trim());
    tf.setEditable(false);
    cpuPanel.add(tf);

    // cpu 6
    l = new JLabel(" Cpu 6: ");
    cpuPanel.add(l);

    tf = new JTextField(detailedInfo.substring(64, 73).trim());
    tf.setEditable(false);
    cpuPanel.add(tf);

    // cpu 7
    l = new JLabel(" Cpu 7: ");
    cpuPanel.add(l);

    tf = new JTextField(detailedInfo.substring(73, 82).trim());
    tf.setEditable(false);
    cpuPanel.add(tf);

    // cpu 8
    l = new JLabel(" Cpu 8: ");
    cpuPanel.add(l);

    tf = new JTextField(detailedInfo.substring(82, 91).trim());
    tf.setEditable(false);
    cpuPanel.add(tf);

    panel1.add(cpuPanel, BorderLayout.SOUTH);

    panel0.add(panel1, BorderLayout.NORTH);

    // Panel containing sound data
    JPanel panel2 = new JPanel(new BorderLayout());

    //Dialog separator
    ds = new DialogSeparator("Sound");
    panel2.add(ds, BorderLayout.NORTH);
    
    JPanel soundPanel = new JPanel(new GridLayout(1, 10));
    
    // Sound 1
    l = new JLabel(" Sound 1: ");
    soundPanel.add(l);

    tf = new JTextField(detailedInfo.substring(91, 103).trim());
    tf.setEditable(false);
    soundPanel.add(tf);
    
    // Sound 2
    l = new JLabel(" Sound 2: ");
    soundPanel.add(l);

    tf = new JTextField(detailedInfo.substring(103, 115).trim());
    tf.setEditable(false);
    soundPanel.add(tf);
    
    // Sound 3
    l = new JLabel(" Sound 3: ");
    soundPanel.add(l);

    tf = new JTextField(detailedInfo.substring(115, 127).trim());
    tf.setEditable(false);
    soundPanel.add(tf);

    // Sound 4
    l = new JLabel(" Sound 4: ");
    soundPanel.add(l);

    tf = new JTextField(detailedInfo.substring(127, 139).trim());
    tf.setEditable(false);
    soundPanel.add(tf);

    // Sound 5
    l = new JLabel(" Sound 5: ");
    soundPanel.add(l);

    tf = new JTextField(detailedInfo.substring(139, 151).trim());
    tf.setEditable(false);
    soundPanel.add(tf);
    
    panel2.add(soundPanel, BorderLayout.SOUTH);
    
    panel0.add(panel2, BorderLayout.CENTER);
    
    // Panel containing other data
    JPanel panel3 = new JPanel(new BorderLayout());

    ds = new DialogSeparator();
    panel3.add(ds, BorderLayout.NORTH);
    
    String extraInfo = "Driver: " + romeo.games.driver + ", " +
                       utils.getString("Popup.listcolors_messagebox_colors") +
                       ": " + colors;
		       
    l = new JLabel(extraInfo);
    
    panel3.add(l, BorderLayout.SOUTH);

    panel0.add(panel3, BorderLayout.SOUTH);

    panel.add(panel0, BorderLayout.NORTH);

    /*
    JPanel panel4 = new JPanel(new BorderLayout());
    
    ds = new DialogSeparator("Clones");
    panel4.add(ds, BorderLayout.NORTH);
    
    JTable table = new JTable();
    
    panel4.add(table, BorderLayout.SOUTH);
    
    panel.add(panel4, BorderLayout.CENTER);
    */

    JInternalFrame jif = new JInternalFrame("Info: " + gameName);

    // set properties
    jif.setClosable(true);
    jif.setMaximizable(false);
    jif.setIconifiable(true);
    jif.setResizable(false);
    jif.setFrameIcon(utils.createImageIcon16x16("Filter.png"));

    jif.setBounds(400, 10, width, height);
    //jif.pack();
    jif.setContentPane(panel);

    desktop.add(jif, layer);

    // Set this internal frame to be selected
    try {
      jif.setSelected(true);
    } catch (java.beans.PropertyVetoException e2) {
    }

    jif.show();

    return jif;
  }
  
  /**
   * FileChooser for history.dat.
   */
  class openHistoryFile implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      fileChooser.setFileFilter(new fileFilter());
      int returnVal = fileChooser.showOpenDialog(Info.this);

      if (returnVal == JFileChooser.APPROVE_OPTION) {
        historyField.setText(fileChooser.getSelectedFile().getPath());

	config = new Config(new File(System.getProperty("user.home") +
	                             File.separator + ".romeo4xmame" + 
				     File.separator + "Romeo.ini"));
        config.add("xmame.history.dat_path", historyField.getText());
        config.save();
      }
    }
  }

  /**
   * Filter for history.dat
   */
  class fileFilter extends FileFilter {
    public boolean accept(File f) {
      if (f.isDirectory())
        return true;
      String path = f.getPath().toLowerCase();
      if (path.endsWith(".dat"))
        return true;
      return false;
    }
    // Return a file description
    public String getDescription() {
      return "History File (*.dat)";
    }
  }
}  

/**
 * The WindowManager class is a simple extension of DefaultDesktopManagerx
 * which overrides all JInternalFrame related methods.
 */
class WindowManager extends DefaultDesktopManager {
  // Custom pager component
  protected WindowWatcher ww;

  /**
   * The WindowManager constructor takes a reference to the desktop it manages, 
   * and in turn passes this reference to the WindowWatcher constructor. 
   * WindowWatcher uses this reference to find out all the information it needs to 
   * know about our desktop?s contents to paint itself correctly .
   *
   * @param desktop The desktop managed by the WindowManager
   */
  public WindowManager(JDesktopPane desktop) {
    ww = new WindowWatcher(desktop);
  }

  /**
   * The getWindowWatcher() method just returns a reference to the WindowWatcher 
   * object, ww, and is used when the desktop is scrolled.
   */
  public WindowWatcher getWindowWatcher() { return ww; }
    
  public void activateFrame(JInternalFrame f) {
    super.activateFrame(f);
    ww.repaint();
  }
  public void beginDraggingFrame(JComponent f) { 
    super.beginDraggingFrame(f);
    ww.repaint();
  }
  public void beginResizingFrame(JComponent f, int direction) {
    super.beginResizingFrame(f,direction);
    ww.repaint();
  }
  public void closeFrame(JInternalFrame f) {
    super.closeFrame(f);
    ww.repaint();
  }
  public void deactivateFrame(JInternalFrame f) {
    super.deactivateFrame(f);
    ww.repaint();
  }
  public void deiconifyFrame(JInternalFrame f) {
    super.deiconifyFrame(f);
    ww.repaint();
  }
  public void dragFrame(JComponent f, int newX, int newY) {
    f.setLocation(newX, newY);
    ww.repaint();
  }
  public void endDraggingFrame(JComponent f) {
    super.endDraggingFrame(f);
    ww.repaint();
  }
  public void endResizingFrame(JComponent f) {
    super.endResizingFrame(f);
    ww.repaint();
  }
  public void iconifyFrame(JInternalFrame f) {
    super.iconifyFrame(f);
    ww.repaint();
  }
  public void maximizeFrame(JInternalFrame f) {
    super.maximizeFrame(f);
    ww.repaint();
  }
  public void minimizeFrame(JInternalFrame f) {
    super.minimizeFrame(f);
    ww.repaint();
  }
  public void openFrame(JInternalFrame f) {
    super.openFrame(f);
    ww.repaint();
  }
  public void resizeFrame(JComponent f,
   int newX, int newY, int newWidth, int newHeight) {
    f.setBounds(newX, newY, newWidth, newHeight);
    ww.repaint();
  }
  public void setBoundsForFrame(JComponent f,
   int newX, int newY, int newWidth, int newHeight) {
    f.setBounds(newX, newY, newWidth, newHeight);
    ww.repaint();
  }
} // That' s all, folks.
