
/*==============================================================================
 *
 * (#)UpdateDialog.java
 * created      2003/08/04
 * version      $Revision: 1.2 $Date: 2003/10/26 01:10:09 $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *==============================================================================
 */

package com.miami.romeo.main;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

import com.miami.romeo.config.*;

/**
 * UpdateDialog.<br>
 * This class creates a dialog box to update the list of
 * games.
 *
 */
public class UpdateDialog extends JDialog {
  Romeo romeo;

  // Utilities
  Utils utils = new Utils();

  // Caption
  String title;

  // Labels for messages
  public JLabel updating    = null;
  public JLabel saving      = null;
  // Progress Bars
  public JProgressBar pBar  = null;
  public JProgressBar pBar2 = null;

  public JDialog dialog;

  /**
   * Constructor.
   *
   * @param frame    The frame to lock
   * @param title    The dialog' s caption
   * @param modal    True if the dialog has to lock the frame parent,
   *                 false otherwise
   */
  public UpdateDialog(JFrame frame, String title, boolean modal) {
    super(frame, title, modal);

    // JPanel panel = new JPanel(new BorderLayout());
    JPanel panel = new SavingDataPanel();

    getContentPane().add(panel, BorderLayout.CENTER);

    setSize(300, 140); // 300, 175 with the cancel button
    setResizable(false);
    Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
    Rectangle frameDim = getBounds();
    setLocation((screenDim.width  - frameDim.width)  / 2,
                (screenDim.height - frameDim.height) / 2);

    show();
  }

  /**
   * Panel for 'Updating list of games' Box
   */
  class SavingDataPanel extends JPanel {
    Romeo romeo;

    /**
     * Constructor.
     *
     * @param class   Romeo    The main class
     */
    public SavingDataPanel() {
      setLayout(null);
      setOpaque(false);

      updating = new JLabel();
      updating.setBounds(10, 10, 270, 20);
      add(updating);

      // This component shows the retrieving data progress
      pBar = new JProgressBar();
      pBar.setBounds(10, 30, 270, 15);
      pBar.setMinimum(0);
      pBar.setStringPainted(true);
      add(pBar);

      saving = new JLabel();
      saving.setBounds(10, 50, 270, 20);
      add(saving);

      // this component shows the saving progress
      pBar2 = new JProgressBar();
      pBar2.setBounds(10, 70, 270, 15);
      pBar2.setMinimum(0);
      pBar2.setStringPainted(true);
      add(pBar2);

      /*
      ImageIcon icon = new ImageIcon(getClass().getResource(
                                                     "/images/icons/24x24/Cancel.png"));

      JButton button = new JButton(utils.getString("UpdateBox.cancel_button_text"));
      button.setIcon(icon);
      button.setBounds(100, 105, 100, 30);
      button.addActionListener(new CancelAction(getDialog()));
      add(button);
      */
    }
  }
  
  /*
  // Cancel button for this box
  class CancelAction extends AbstractAction {
    JDialog dialog;

    protected CancelAction(JDialog dialog) {
      super("CancelAction");
      this.dialog = dialog;
    }

    public void actionPerformed(ActionEvent e) {
      if (bw == null) {
        dialog.setVisible(false);
        dialog = null;
      }
      else
        return;
    }
  }
  */

  /**
   * This method sets the messages in a given label.
   *
   * @param label The label to set the messages in
   * @param str   The message
   */
  public void message(final JLabel label, final String str) {
    if (str != null) {
      Runnable runner = new Runnable() {
        public void run() {
          label.setText(str);
        }
      };
      SwingUtilities.invokeLater(runner);
    }
  }

  /**
   * This method sets the progress bar' s current value to int value.
   *
   * @param progressBar The progress bar
   * @param value       The value to set
   */
  public void setProgressBarValue(final JProgressBar progressBar,
                                  final int value) {
    Runnable runner = new Runnable() {
      public void run() {
        progressBar.setValue(value);
      }
    };
    SwingUtilities.invokeLater(runner);
  }

  /**
   *  This method sets the progress bar' s maximum value to int value.
   *
   * @param progressBar The progress bar
   * @param value       The value to set
   */
  public void setProgressBarMaximum(final JProgressBar progressBar,
                                    final int value) {
    Runnable runner = new Runnable() {
      public void run() {
        progressBar.setMaximum(value);
      }
    };
    SwingUtilities.invokeLater(runner);
  }

  /**
   *  This method sets the value of the progress string.
   *
   * @param progressBar The progress bar
   * @param string      The value of the progress string
   */
  public void setProgressBarString(final JProgressBar progressBar,
                                   final String string) {
    Runnable runner = new Runnable() {
      public void run() {
        progressBar.setString(string);
      }
    };
    SwingUtilities.invokeLater(runner);
  }
  
  /**
   * This method returns the dialog window.
   *
   * @return <code>JDialog</code>
   */
  public JDialog getDialog() {
    return this;
  }
}
